@@ -7,62 +7,33 @@
 #if defined(__STDC__)
 #include <stdlib.h>
 #endif
+
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 #include "preferences.h"
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
-
-extern void anomaly P_((double ma, double s, double *nu, double *ea));
-extern void comet P_((double Mjd, double ep, double inc, double ap, double qp,
-    double om, double *lpd, double *psi, double *rp, double *rho, double *lam,
-    double *bet));
-extern void ecl_eq P_((double Mjd, double lati, double lngi, double *ra,
-    double *dec));
-extern void eq_ecl P_((double Mjd, double ra, double dec, double *lati,
-    double *lngi));
-extern void gk_mag P_((double g, double k, double rp, double rho, double *mp));
-extern void hadec_aa P_((double lati, double ha, double dec, double *alt,
-    double *az));
+extern int obj_earthsat P_((Now *np, Obj *op));
+extern void now_lst P_((Now *np, double *lst));
 extern void hg_mag P_((double h, double g, double rp, double rho, double rsn,
     double *mp));
-extern void moon P_((double Mjd, double *lam, double *bet, double *hp));
-extern void now_lst P_((Now *np, double *lst));
-extern void nutation P_((double Mjd, double *deps, double *dpsi));
-extern void plans P_((double Mjd, int p, double *lpd0, double *psi0,
-    double *rp0, double *rho0, double *lam, double *bet, double *dia,
-    double *mag));
-extern void precess P_((double mjd1, double mjd2, double *ra, double *dec));
-extern void range P_((double *v, double r));
-extern void reduce_elements P_((double mjd0, double Mjd, double inc0,
-    double ap0, double om0, double *inc, double *ap, double *om));
-extern void refract P_((double pr, double tr, double ta, double *aa));
-extern void sunpos P_((double Mjd, double *lsn, double *rsn));
-extern void ta_par P_((double tha, double tdec, double phi, double ht,
-    double ehp, double *aha, double *adec));
-extern obj_earthsat P_((Now *np, Obj *op));
+extern void gk_mag P_((double g, double k, double rp, double rho, double *mp));
 
-int obj_cir P_((Now *np, Obj *op));
-static obj_planet P_((Now *np, Obj *op));
-static obj_fixed P_((Now *np, Obj *op));
-static obj_elliptical P_((Now *np, Obj *op));
-static obj_hyperbolic P_((Now *np, Obj *op));
-static obj_parabolic P_((Now *np, Obj *op));
-static sun_cir P_((Now *np, Obj *op));
-static moon_cir P_((Now *np, Obj *op));
+static int obj_planet P_((Now *np, Obj *op));
+static int obj_fixed P_((Now *np, Obj *op));
+static int obj_elliptical P_((Now *np, Obj *op));
+static int obj_hyperbolic P_((Now *np, Obj *op));
+static int obj_parabolic P_((Now *np, Obj *op));
+static int sun_cir P_((Now *np, Obj *op));
+static int moon_cir P_((Now *np, Obj *op));
 static void cir_sky P_((Now *np, double lpd, double psi, double rp, double rho,
     double lam, double bet, double lsn, double rsn, Obj *op));
 static void elongation P_((double lam, double bet, double lsn, double *el));
 
-#undef P_
-
 /* given a Now and an Obj, fill in the approprirate s_* fields within Obj.
  * return 0 if all ok, else -1.
  */
+int
 obj_cir (np, op)
 Now *np;
 Obj *op;
@@ -81,7 +52,7 @@ Obj *op;
 	}
 }
 
-static
+static int
 obj_planet (np, op)
 Now *np;
 Obj *op;
@@ -126,7 +97,7 @@ Obj *op;
 	return (0);
 }
 
-static
+static int
 obj_fixed (np, op)
 Now *np;
 Obj *op;
@@ -194,7 +165,7 @@ Obj *op;
 
 /* compute sky circumstances of an object in heliocentric elliptic orbit at *np.
  */
-static
+static int
 obj_elliptical (np, op)
 Now *np;
 Obj *op;
@@ -301,7 +272,7 @@ Obj *op;
 
 /* compute sky circumstances of an object in heliocentric hyperbolic orbit.
  */
-static
+static int
 obj_hyperbolic (np, op)
 Now *np;
 Obj *op;
@@ -394,7 +365,7 @@ Obj *op;
 
 /* compute sky circumstances of an object in heliocentric hyperbolic orbit.
  */
-static
+static int
 obj_parabolic (np, op)
 Now *np;
 Obj *op;
@@ -434,7 +405,7 @@ Obj *op;
 
 /* find sun's circumstances now.
  */
-static
+static int
 sun_cir (np, op)
 Now *np;
 Obj *op;
@@ -482,7 +453,7 @@ Obj *op;
 	/* find alt/az based on unprecessed ra/dec */
 	now_lst (np, &lst);
 	ha = hrrad(lst) - ra;
-	ehp = (2.0 * 6378.0 / 146.0e6) / op->s_edist;
+	ehp = (2.0 * ERAD / MAU) / op->s_edist;
 	ta_par (ha, dec, lat, elev, ehp, &ha, &dec);
 	hadec_aa (lat, ha, dec, &alt, &az);
 	refract (pressure, temp, alt, &alt);
@@ -493,7 +464,7 @@ Obj *op;
 
 /* find moon's circumstances now.
  */
-static
+static int
 moon_cir (np, op)
 Now *np;
 Obj *op;
@@ -515,7 +486,7 @@ Obj *op;
 	lam += dpsi;
 	range (&lam, 2*PI);
 
-	op->s_edist = 6378.14/sin(ehp);	/* earth-moon dist, want km */
+	op->s_edist = (ERAD/MAU)/sin(ehp);	/* earth-moon dist, want au */
 	op->s_size = 3600*31.22512*sin(ehp);/* moon angular dia, seconds */
 	op->s_hlong = lam;		/* save geo in helio fields */
 	op->s_hlat = bet;
@@ -539,7 +510,7 @@ Obj *op;
 	elongation (lam, bet, lsn, &el);
 
 	/* solve triangle of earth, sun, and elongation for moon-sun dist */
-	edistau = op->s_edist/1.495979e8;	/* km -> au */
+	edistau = op->s_edist;
 	op->s_sdist= sqrt (edistau*edistau + rsn*rsn - 2.0*edistau*rsn*cos(el));
 
 	/* TODO: improve mag; this is based on a flat moon model. */
@@ -652,5 +623,5 @@ double lam, bet, lsn;
 double *el;
 {
 	*el = acos(cos(bet)*cos(lam-lsn));
-	if (lam>lsn+PI || lam>lsn-PI && lam<lsn) *el = - *el;
+	if (lam>lsn+PI || (lam>lsn-PI && lam<lsn)) *el = - *el;
 }
