@@ -14,54 +14,46 @@
 #include <Xm/Frame.h>
 #include <Xm/Label.h>
 #include <Xm/PushB.h>
+#include <Xm/TextF.h>
 #include <Xm/ToggleB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/Separator.h>
+
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 #include "preferences.h"
 
 extern Widget	toplevel_w;
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
-
-extern double mjd_hr P_((double jd));
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
+extern char *cns_name P_((int id));
+extern double atod P_((char *buf));
 extern int any_ison P_((void));
-extern void confnd P_((double r, double d, double e, char **name));
+extern int cns_pick P_((double ra, double dec, double e));
 extern void db_update P_((Obj *op));
+extern void dm_update P_((Now *np, int how_much));
 extern void f_dm_angle P_((Widget w, double a));
-extern void f_pangle P_((Widget w, double a));
 extern void f_double P_((Widget w, char *fmt, double f));
 extern void f_mtime P_((Widget w, double t));
+extern void f_pangle P_((Widget w, double a));
 extern void f_ra P_((Widget w, double ra));
 extern void f_showit P_((Widget w, char *s));
 extern void f_string P_((Widget w, char *s));
-extern void fs_mtime P_((char out[], double t));
+extern void f_time P_((Widget w, double t));
 extern void field_log P_((Widget w, double value, int logv, char *str));
-extern void get_something P_((Widget w, char *resource, char *value));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
+extern void fs_mtime P_((char out[], double t));
+extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void register_selection P_((char *name));
 extern void riset_cir P_((Now *np, Obj *op, double dis, RiseSet *rp));
-extern void set_something P_((Widget w, char *resource, char *value));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void timestamp P_((Now *np, Widget w));
-extern void unrefract P_((double pr, double tr, double aa, double *ta));
 extern void watch_cursor P_((int want));
 extern void xe_msg P_((char *msg, int app_modal));
 
-void dm_manage P_((void));
-void dm_newobj P_((int dbidx));
-void dm_update P_((Now *np, int how_much));
-void dm_selection_mode P_((int whether));
-void dm_riset P_((Now *np, Obj *op, RiseSet *rsp));
-void dm_cursor P_((Cursor c));
 static void dm_create_form P_((void));
 static void dm_set_buttons P_((int whether));
 static void dm_create_table P_((Widget parent));
@@ -71,7 +63,8 @@ static void dm_setup_cb P_((Widget w, XtPointer client, XtPointer call));
 static void dm_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void dm_compute P_((int r, int force, Now *np));
 static void dm_format P_((Now *np, Obj *op, RiseSet *rp, int c, Widget w));
-static void dm_rs_addplus P_((Widget w, double tm, int addplus));
+static void dm_addplus P_((Now *np, Widget w, double tm, int tzpref,
+    int addplus));
 static void dm_rs_hrsup P_((Now *np, Obj *op, Widget w, RiseSet *rp));
 static void show_constellation P_((Now *np, Obj *op, Widget w));
 static void dm_separation P_((Obj *p, Obj *q, int how, Widget w));
@@ -81,6 +74,7 @@ static void ds_setup_col_selections P_((int what));
 static void ds_apply_selections P_((void));
 static void ds_ctl_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ds_help P_((void));
+static void ds_refr_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ds_row_toggle_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ds_col_toggle_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ds_row_all_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -88,8 +82,6 @@ static void ds_col_all_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ds_row_reset_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ds_col_reset_cb P_((Widget w, XtPointer client, XtPointer call));
 
-#undef P_
-
 typedef struct {
     int dbidx;		/* db index of object on this row */
     int on;		/* whether this row is currently to be on */
@@ -131,8 +123,8 @@ enum {
  */
 enum {
     CONSTEL_ID, RA_ID, DEC_ID, AZ_ID, ALT_ID, HLONG_ID, HLAT_ID,
-    EDST_ID, SDST_ID, ELONG_ID, SIZE_ID, VMAG_ID, PHS_ID, RSTIME_ID,
-    RSAZ_ID, TRTIME_ID, TRALT_ID, SETTIME_ID, SETAZ_ID, HRSUP_ID
+    EDST_ID, ELGHT_ID, SDST_ID, SLGHT_ID, ELONG_ID, SIZE_ID, VMAG_ID, PHS_ID,
+    RSTIME_ID, RSAZ_ID, TRTIME_ID, TRALT_ID, SETTIME_ID, SETAZ_ID, HRSUP_ID
 };
 
 /* tags for the various Data Selection control panel buttons */
@@ -147,7 +139,9 @@ static ColHdr col[] = {
     {MISC_COL,	"HeLong"},
     {MISC_COL,	"HeLat"},
     {MISC_COL,	"EaDst"},
+    {MISC_COL,	"EaLght"},
     {MISC_COL,	"SnDst"},
+    {MISC_COL,	"SnLght"},
     {MISC_COL,	"Elong"},
     {MISC_COL,	"Size"},
     {MISC_COL,	"VMag"},
@@ -191,6 +185,7 @@ enum {STDREFR, ADPREFR};
 enum {LIMB, CENTER};
 
 static Widget stdrefr_w;	/* the StdRefr toggle button */
+static Widget adprefr_w;	/* the text holding the adaptive ref horizon */
 static Widget limb_w;		/* the Center/Limb toggle button */
 static Widget refr_w;		/* the horizon label on the data table */
 static Widget limbl_w;		/* the Limb label on the data table */
@@ -319,8 +314,8 @@ int whether;	/* whether setting up for plotting or for not plotting */
 	dm_selecting += whether ? 1 : -1;
 
 	if (dataform_w && XtIsManaged(dataform_w))
-	    if (whether && dm_selecting == 1     /* first one to want on */
-		|| !whether && dm_selecting == 0 /* last one to want off */)
+	    if ((whether && dm_selecting == 1)     /* first one to want on */
+		|| (!whether && dm_selecting == 0) /* last one to want off */)
 		dm_set_buttons (whether);
 }
 
@@ -333,18 +328,22 @@ Now *np;
 Obj *op;
 RiseSet *rsp;
 {
-	double dis = 0;	/* rads apparent horizon is from true */
+	double dis;	/* rads apparent horizon is above true */
 	int close = is_planet(op,MOON) || is_planet(op,SUN); 
 
 	/* refraction correction */
 	if (horizon == STDREFR) {
 	    /* `nominal' atmospheric refraction. */
-	    dis += STDREF;
+	    dis = STDREF;
 	} else {
 	    /* `Adaptive:' actual refraction conditions  */
-	    double ref;
-	    unrefract (pressure, temp, 0.0, &ref);
-	    dis -= ref;	/* downwards */
+	    char *str;
+
+	    str = XmTextFieldGetString (adprefr_w);
+	    dis = degrad(atod (str));
+	    XtFree (str);
+	    unrefract (pressure, temp, dis, &dis);
+	    dis = -dis;	/* downwards */
 	}
 
 	/* add object's semi-diameter if want upper limb.
@@ -696,11 +695,11 @@ XtPointer call;
 	    (void) sprintf (name, "%s.%s", rname, cname);
 
 	    /* set XmNuserData to be the name we want to go by */
-	    get_something (w, XmNuserData, (char *)&userD);
+	    get_something (w, XmNuserData, (XtArgVal)&userD);
 	    if (userD)
 		XtFree (userD);
 	    userD = name;
-	    set_something (w, XmNuserData, userD);
+	    set_something (w, XmNuserData, (XtArgVal)userD);
 
 	    /* tell the world our name */
 	    register_selection (name);
@@ -835,8 +834,11 @@ Widget w;
 	    if (is_planet(op, MOON)) {
 		double tmp = op->s_edist;
 		if (pref_get(PREF_UNITS) == PREF_ENGLISH) {
-		    /* s_edist is stored in km, show in miles */
-		    tmp /= 1.609344;
+		    /* s_edist is stored in au, want miles */
+		    tmp *= MAU*FTPM/5280.0;
+		} else {
+		    /* s_edist is stored in au, want km */
+		    tmp *= MAU/1000.0;
 		}
 		f_double (w, "%6.0f", tmp);
 	    } else if (is_ssobj(op)) {
@@ -846,6 +848,26 @@ Widget w;
 	    } else
 		f_string (w, "      ");
 	    break;
+	case ELGHT_ID:
+	    if (is_planet(op,MOON)) {
+	        double m = op->s_edist*LTAU;	/* seconds */
+		if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
+		    f_double (w, "%5.3f", m);
+		else
+		    f_double (w, "% 7.5f", m);	/* 8.6 is ridiculous */
+	    } else if (is_ssobj(op)) {
+	        double m = (op->s_edist*LTAU)/3600.0;	/* hours */
+		if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
+		    f_mtime (w, m);	/* hh:mm */
+		else
+		    f_time (w, m);	/* hh:mm:ss */
+	    } else {
+		if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
+		    f_string (w, "     ");
+		else
+		    f_string (w, "        ");
+	    }
+	    break;
 	case SDST_ID:
 	    if (is_ssobj(op) && !is_planet(op, SUN))
 		f_double (w, op->s_sdist >= 9.99995 ? "%6.3f" : "%6.4f",
@@ -853,6 +875,20 @@ Widget w;
 	    else
 		f_string (w, "      ");
 	    break;
+	case SLGHT_ID:
+	    if (is_ssobj(op) && !is_planet(op, SUN)) {
+	        double m = (op->s_sdist*LTAU)/3600.0;	/* hours */
+		if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
+		    f_mtime (w, m);	/* hh:mm */
+		else
+		    f_time (w, m);	/* hh:mm:ss */
+	    } else {
+		if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
+		    f_string (w, "     ");
+		else
+		    f_string (w, "        ");
+	    }
+	    break;
 	case ELONG_ID:
 	    if (is_ssobj(op) && !is_planet(op, SUN))
 		f_double (w, "%6.1f", op->s_elong);
@@ -861,9 +897,9 @@ Widget w;
 	    break;
 	case SIZE_ID:
 	    if (op->type != EARTHSAT)
-		f_double (w, "%4.0f", (double)(op->s_size));
+		f_double (w, "%5.0f", (double)(op->s_size));
 	    else
-		f_string (w, "    ");
+		f_string (w, "     ");
 	    break;
 	case VMAG_ID:
 	    if (op->type != EARTHSAT) {
@@ -889,7 +925,7 @@ Widget w;
 	    else if (rp->rs_flags & RS_NORISE)
 		f_string (w, "NoRise");
 	    else
-		dm_rs_addplus (w, rp->rs_risetm, rp->rs_flags & RS_2RISES);
+		dm_addplus (np, w, rp->rs_risetm, 1, rp->rs_flags & RS_2RISES);
 	    break;
 
 	case RSAZ_ID:
@@ -915,7 +951,7 @@ Widget w;
 	    else if (rp->rs_flags & RS_NOSET)
 		f_string (w, "NoSet ");
 	    else
-		dm_rs_addplus (w, rp->rs_settm, rp->rs_flags & RS_2SETS);
+		dm_addplus (np, w, rp->rs_settm, 1, rp->rs_flags & RS_2SETS);
 	    break;
 
 	case SETAZ_ID:
@@ -939,7 +975,7 @@ Widget w;
 	    else if (rp->rs_flags & RS_NOTRANS)
 		f_string (w, "NoTran");
 	    else
-		dm_rs_addplus (w, rp->rs_trantm, rp->rs_flags & RS_2TRANS);
+		dm_addplus (np, w, rp->rs_trantm, 1, rp->rs_flags & RS_2TRANS);
 	    break;
 
 	case TRALT_ID:
@@ -979,16 +1015,24 @@ Widget w;
 
 /* display the rise/set/transit time tm (in hours) in widget w.
  * if addplus is != 0 then add a '+' to the time string.
+ * tm is hours; convert to UTC if tzpref && PREF_ZONE == PREF_UTCTZ.
  * N.B. be sure the total string length is the same regardless of addplus.
  */
 static void
-dm_rs_addplus (w, tm, addplus)
+dm_addplus (np, w, tm, tzpref, addplus)
+Now *np;
 Widget w;
 double tm;
+int tzpref;
 int addplus;
 {
 	char str[32];
 
+	if (tzpref && pref_get(PREF_ZONE) == PREF_UTCTZ) {
+	    tm += tz;
+	    range (&tm, 24.0);
+	}
+
 	fs_mtime (str, tm);				/* 5 chars: "hh:mm" */
 	(void) strcat (str, addplus ? "+" : " ");	/* always add 1 */
 	field_log (w, tm, 1, str);
@@ -1023,7 +1067,7 @@ RiseSet *rp;
 	    return;
 	}
 	if (rp->rs_flags & RS_NEVERUP) {
-	    dm_rs_addplus (w, 0.0, 0);
+	    dm_addplus (np, w, 0.0, 0, 0);
 	    return;
 	}
 
@@ -1045,7 +1089,7 @@ RiseSet *rp;
 
 	    /* only if set time is really after rise time can we figure hrsup */
 	    if (s > r)
-		dm_rs_addplus(w, s - r, 0);
+		dm_addplus(np, w, s - r, 0, 0);
 	    else
 		f_string (w, "      ");
 
@@ -1058,7 +1102,7 @@ RiseSet *rp;
 	if (hrs < 0)
 	    hrs += 24.0;
 
-	dm_rs_addplus(w, hrs,
+	dm_addplus(np, w, hrs, 0,
 			rp->rs_flags&(RS_NORISE|RS_NOSET|RS_2RISES|RS_2SETS));
 }
 
@@ -1069,8 +1113,10 @@ Obj *op;
 Widget w;
 {
 	char nm[10], *name;
+	int id;
 
-        confnd (op->s_ra, op->s_dec, epoch == EOD ? mjd : epoch, &name);
+        id = cns_pick (op->s_ra, op->s_dec, epoch == EOD ? mjd : epoch);
+	name = cns_name (id);
 	(void) sprintf (nm, "%.3s", name);
 	f_string(w, nm);
 }
@@ -1128,7 +1174,7 @@ Widget parent;
 	Widget rl_w, cl_w;
 	Widget rf_w, mf_w, rsf_w, sf_w, ctlf_w, rowrc_w, mrc_w, rsrc_w, src_w;
 	Widget sep_w, w, tb1, tb2;
-	Widget rb_w;
+	Widget rb_w, rc_w;
 	int n;
 	int i;
 
@@ -1375,27 +1421,47 @@ Widget parent;
 							(XtPointer)RISET_COL);
 	    XtManageChild (w);
 
-	    /* make the STDREFR/ADPREFR radio box */
+	    /* make the STDREFR/ADPREFR radio box and horizon offset text */
 
 	    n = 0;
 	    w = XmCreateFrame (rsrc_w, "DSRefrF", args, n);
 	    XtManageChild (w);
 	    n = 0;
-	    rb_w = XmCreateRadioBox (w, "DSRefrRB", args, n);
-	    XtManageChild (rb_w);
+	    rc_w = XmCreateRowColumn (w, "DSRefrRC", args, n);
+	    XtManageChild (rc_w);
 
 		n = 0;
-		stdrefr_w = tb1 = XmCreateToggleButton (rb_w, "StdRefr",args,n);
-		XtManageChild (tb1);
+		rb_w = XmCreateRadioBox (rc_w, "DSRefrRB", args, n);
+		XtManageChild (rb_w);
+
+		    n = 0;
+		    tb1 = XmCreateToggleButton (rb_w, "StdRefr", args,n);
+		    XtAddCallback (tb1, XmNvalueChangedCallback, ds_refr_cb, 0);
+		    XtManageChild (tb1);
+		    stdrefr_w = tb1;
+
+		    n = 0;
+		    tb2 = XmCreateToggleButton (rb_w, "AdpRefr", args, n);
+		    XtManageChild (tb2);
+
 		n = 0;
-		tb2 = XmCreateToggleButton (rb_w, "AdpRefr", args, n);
-		XtManageChild (tb2);
-		/* if neither or both is set up in defaults, set for StdRefr */
+		adprefr_w = XmCreateTextField (rc_w, "AdpDispl", args, n);
+		XtManageChild (adprefr_w);
+
+		/* if neither or both is set up in defaults, set for StdRefr
+		 * otherwise just trigger one to set the text field.
+		 */
 		if (XmToggleButtonGetState(tb1) == XmToggleButtonGetState(tb2)){
 		    xe_msg (
-		     "Conflicting Refraction resources -- defaulting to StdRefr\n",
+		    "Conflicting Refraction resources -- defaulting to StdRefr",
 									    0);
-		    XmToggleButtonSetState (tb1, True, True);
+		    XmToggleButtonSetState (stdrefr_w, True, True);
+		} else {
+		    /* must toggle state to insure callbacks really called */
+		    XmToggleButtonSetState (stdrefr_w,
+				    !XmToggleButtonGetState(stdrefr_w), True);
+		    XmToggleButtonSetState (stdrefr_w,
+				    XmToggleButtonGetState(stdrefr_w), True);
 		}
 
 	    /* make the CENTER/LIMB radio box */
@@ -1593,7 +1659,7 @@ ds_apply_selections()
 
 	watch_cursor(1);
 
-	if (wasman = XtIsManaged(dataform_w))
+	if ((wasman = XtIsManaged(dataform_w)))
 	    XtUnmanageChild (dataform_w);
 
 	for (i = 0; i < NR; i++) {
@@ -1697,10 +1763,22 @@ ds_help ()
 	hlp_dialog ("DataSelection Table", msg, XtNumber(msg));
 }
 
+/* callback from the stdrefr_w tb.
+ */
+/* ARGSUSED */
+static void
+ds_refr_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	XtSetSensitive (adprefr_w, !XmToggleButtonGetState(w));
+}
+
 /* callback from the Data selection row toggle button.
  */
-static void
 /* ARGSUSED */
+static void
 ds_row_toggle_cb (w, client, call)
 Widget w;
 XtPointer client;
