@@ -12,71 +12,71 @@
 extern void *malloc(), *realloc();
 #endif
 
+#if defined(_POSIX_SOURCE)
+#include <unistd.h>
+#else
+extern int read();
+extern int close();
+#endif
+
 #include <X11/Intrinsic.h>
 
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 #include "preferences.h"
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
-
-extern char *syserrstr P_((void));
-extern int openh P_((char *name, int flags));
-extern double atod P_((char *buf));
-extern Now *mm_get_now P_((void));
-extern int obj_cir P_((Now *np, Obj *op));
-extern void all_newdb P_((int appended));
-extern void cal_mjd P_((int mn, double dy, int yr, double *Mjd));
-extern void f_dec_sexsign P_((double x, int *h, int *m, int *s));
-extern void f_sscandate P_((char *bp, int pref, int *m, double *d, int *y));
-extern void f_sscansex P_((char *bp, int *d, int *m, int *s));
-extern void mjd_cal P_((double Mjd, int *mn, double *dy, int *yr));
-extern void sex_dec P_((int hd, int m, int s, double *d));
-extern void watch_cursor P_((int want));
-extern void xe_msg P_((char *msg, int app_modal));
-extern void zero_mem P_((char *loc, unsigned len));
-
-int db_n P_((void));
-int db_n_cp P_((void));
-Obj *db_basic P_((int id));
-void db_setuserobj P_((int id, Obj *op));
-void db_invalidate P_((void));
-void db_scaninit P_((DBScan *sp, HowScan how));
-Obj *db_scan P_((DBScan *sp));
-void db_update P_((Obj *op));
-void db_del_all P_((void));
-void db_del_cp P_((void));
-void db_set_cp P_((void));
-int db_read P_((FILE *fp));
-int db_set_field P_((char bp[], int id, PrefDateFormat pref, Obj *op));
-void db_connect_fifo P_((void));
-static int db_crack_line P_((char *s));
-static Obj *db_new P_((int t));
-static void db_init P_((void));
-static int nxt_db P_((char buf[], int blen, FILE *fp));
-static int line_candidate P_((char *buf));
-static void crack_year P_((char *bp, PrefDateFormat pref, double *p));
-/* don't know why but my compiler complains with this one in here
-static get_fields P_((char *s, char delim, char *fields[]));
-*/
-static get_fields ();
-static void dbfifo_cb P_((XtPointer client, int *fdp, XtInputId *idp));
-
-#undef P_
-
 extern Widget toplevel_w;
 extern char *myclass;
 extern XtAppContext xe_app;
 #define XtD XtDisplay(toplevel_w)
 
+
+extern FILE *fopenh P_((char *name, char *how));
+extern Now *mm_get_now P_((void));
+extern char *syserrstr P_((void));
+extern double atod P_((char *buf));
+extern int db_n P_((void));
+extern int db_n_cp P_((void));
+extern int db_read P_((FILE *fp));
+extern int db_set_field P_((char bp[], int id, PrefDateFormat pref, Obj *op));
+extern int obj_cir P_((Now *np, Obj *op));
+extern int openh P_((char *name, int flags));
+extern void all_newdb P_((int appended));
+extern void db_connect_fifo P_((void));
+extern void db_del_all P_((void));
+extern void db_del_cp P_((void));
+extern void db_scaninit P_((DBScan *sp));
+extern void db_set_cp P_((void));
+extern void db_clr_cp P_((void));
+extern void db_update P_((Obj *op));
+extern void f_scansex P_((double o, char bp[], double *np));
+extern void f_sscandate P_((char *bp, int pref, int *m, double *d, int *y));
+extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
+extern void pm_set P_((int p));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
+extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void watch_cursor P_((int want));
+extern void xe_msg P_((char *msg, int app_modal));
+extern void zero_mem P_((void *loc, unsigned len));
+
+
+static int db_crack_line P_((char s[]));
+static Obj *db_new P_((int t));
+static void db_init P_((void));
+static void db_nodups P_((void));
+static int nxt_db P_((char buf[], int blen, FILE *fp));
+static int line_candidate P_((char *buf));
+static void crack_year P_((char *bp, PrefDateFormat pref, double *p));
+static int get_fields P_((char *s, int delim, char *fields[]));
+static void dbfifo_cb P_((XtPointer client, int *fdp, XtInputId *idp));
+
 #define	MAXDBLINE	256	/* longest allowed db line */
 #define	FLDSEP		','	/* major field separator */
 #define	SUBFLD		'|'	/* subfield separator */
 
+#define	DBFIFO_MSG	'!'	/* introduces a message line from the DBFIFO */
+
 #define	ASIZ(a)	(sizeof(a)/sizeof(a[0]))
 
 /* This counter is incremented when we want to mark all the Obj derived entries
@@ -89,7 +89,7 @@ static Objage_t db_age;
 
 /* the "database".
  * one such struct per object type. the space for objects is malloced in
- *   chunks of DBCHUNK more units for efficiency.
+ *   seperate chunks of DBCHUNK to avoid needing big contiguous memory blocks.
  * N.B. because the number is fixed and known, we use static storage for the
  *   NOBJ Objects for the PLANET type; see db_init().
  */
@@ -111,6 +111,17 @@ static DBMem db[NOBJTYPES];	/* this is the head for each object */
 #define	OBJP(t,n)	\
 		((Obj *)(db[t].dblist[(n)/DBCHUNK] + ((n)%DBCHUNK)*db[t].size))
 
+/* db fifo fd and XtAddInput id.
+ */
+static int db_fifofd = -1;
+static XtInputId db_fifoid;
+
+
+/* set whenever we want to take care to eliminate any items in the db with
+ *   duplicate names.
+ */
+static int db_wantnodups;
+
 /* return number of objects in the database.
  * this includes the NOBJ basic objects.
  * N.B. this is expected to be inexpensive to call.
@@ -129,9 +140,10 @@ db_n()
 	return (n);
 }
 
-/* return the total current number of objects as of the last checkpoint 
+/* return the total current number of objects as of the last checkpoint.
  * this includes the NOBJ basic objects.
  */
+int
 db_n_cp()
 {
 	DBMem *dmp;
@@ -145,8 +157,8 @@ db_n_cp()
 	return (n);
 }
 
-/* given one of the basic ids in astro.h return pointer to its Obj in the
- * database.
+/* given one of the basic ids in astro.h return pointer to its updated Obj in
+ * the database.
  */
 Obj *
 db_basic(id)
@@ -168,6 +180,83 @@ int id;
 	return (op);
 }
 
+/* load each file of objects listed in the DBinitialFiles resource
+ * and inform all modules of the update.
+ */
+void
+db_loadinitial()
+{
+	static char dbd[] = "DBdirectory";	/* dir resource name */
+	static char dbi[] = "DBinitialFiles";	/* init files resource name */
+	char *fns;		/* value of DBinitialFiles */
+	char *dir;		/* value of DBdirectory */
+	char dbicpy[2048];	/* local copy of dir */
+	char *fnf[128];		/* ptrs into dbicpy[] at each ' ' */
+	char fn[1024];		/* each dir/filename */
+	char msg[256];
+	FILE *fp;
+	int nfn;
+	int i;
+
+	/* get the initial directory name */
+	dir = XGetDefault (XtD, myclass, dbd);
+	if (!dir) {
+	    dir = ".";
+	    (void) sprintf (msg, "Can not get %.100s.%.100s -- using %s",
+							    myclass, dbd, dir);
+	    xe_msg (msg, 0);
+	}
+
+	/* get the initial list of files */
+	fns = XGetDefault (XtD, myclass, dbi);
+	if (!fns) {
+	    fns = "basic.edb";
+	    (void) sprintf (msg ,"Can not get %.100s.%.100s -- using %s",
+							myclass, dbi, fns);
+	    xe_msg (msg, 0);
+	}
+
+	/* work on a copy since we are about to break into fields */
+	dbicpy[sizeof(dbicpy)-1] = '\0';
+	(void) strncpy (dbicpy, fns, sizeof(dbicpy)-1);
+	nfn = get_fields (dbicpy, ' ', fnf);
+	if (nfn > XtNumber(fnf)) {
+	    /* we exit because we've clobbered our stack by now!
+	     * TODO: pass the size of fnf to get_fields().
+	     */
+	    printf ("Too many entries in %s. Max is %d\n", dbi, XtNumber(fnf));
+	    exit (1);
+	}
+
+	/* open and read each file.
+	 * N.B. get_fields() will return 1 even if there are no fields.
+	 */
+	for (i = 0; i < nfn && fnf[i][0] != '\0'; i++) {
+#ifdef VMS
+	    /* VMS doesn't like the /'s between components.
+	     * this from Harry Payne, payne@stsci.edu
+	     */
+	    (void) sprintf (fn, "%.500s%.500s", dir, fnf[i]);
+#else
+	    (void) sprintf (fn, "%.500s/%.500s", dir, fnf[i]);
+#endif
+	    fp = fopenh (fn, "r");
+	    if (!fp) {
+		(void) sprintf (msg, "Can not open %.150s: %.50s\n", fn,
+								syserrstr());
+		xe_msg (msg, 1);
+		continue;
+	    }
+	    if (db_read (fp) < 0) {
+		(void) sprintf (msg, "Error reading `%.200s'", fn);
+		xe_msg (msg, 0);
+	    }
+	    (void) fclose (fp);
+	}
+
+	all_newdb(1);
+}
+
 /* set our official copy of the given user-defined object to *op */
 void
 db_setuserobj(id, op)
@@ -194,23 +283,24 @@ db_invalidate()
 	db_age++;
 }
 
-/* initialize the given DBScan for a certain type of database scan.
- * the idea is to call this once, the repeatedly call db_scan() to get all,
- * or a certain subset, of the objects in the db, though in no particular order.
- * return NULL if there are no more.
+/* initialize the given DBScan for a database scan. the idea is to call this
+ *   once, then repeatedly call db_scan() to get all objects in the db.
+ * return NULL when there are no more.
  * N.B. nothing should be assumed as to the order these are returned.
- * N.B. the s_ fields are *not* updated -- all db_update() when you need that.
+ * N.B. the s_ fields are *not* updated -- call db_update() when you need that.
  */
 void
-db_scaninit (sp, how)
+db_scaninit (sp)
 DBScan *sp;
-HowScan how;
 {
-	sp->how = how;
 	sp->t = 0;
 	sp->n = 0;
 }
 
+/* fetch the next object.
+ * N.B. this will return the user defined objects which may still
+ *   have op->type == UNDEFOBJ.
+ */
 Obj *
 db_scan (sp)
 DBScan *sp;
@@ -220,8 +310,7 @@ DBScan *sp;
 	if (!DBINITED)
 	    db_init();
 
-	while ((sp->how == SCAN_ALLBUT_PLANET && sp->t == PLANET) ||
-						    sp->n >= db[sp->t].nobj) {
+	while (sp->n >= db[sp->t].nobj) {
 	    if (++sp->t == NOBJTYPES)
 		return (0);
 	    sp->n = 0;
@@ -253,7 +342,7 @@ Obj *op;
 	    printf ("%s: called with UNDEFOBJ pointer\n", me);
 	    exit (1);
 	} 
-	if (op->type <= 0 || op->type >= NOBJTYPES) {
+	if ((int)op->type >= NOBJTYPES) {
 	    printf ("%s: called with bad pointer\n", me);
 	    exit (1);
 	} 
@@ -296,6 +385,7 @@ db_del_all()
 }
 
 /* delete all objects back to the last checkpoint.
+ * we don't actually free memory.
  */
 void
 db_del_cp ()
@@ -323,35 +413,97 @@ db_set_cp()
 	    dmp->ncp = dmp->nobj;
 }
 
+/* set the checkpoints to 0 then, if active, also eliminate dups.
+ * N.B. but not the PLANT one.
+ */
+void
+db_clr_cp()
+{
+	DBMem *dmp;
+
+	if (!DBINITED)
+	    db_init();
+
+	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++) {
+	    /* N.B. PLANET entries are fixed -- not malloced */
+	    if (dmp != &db[PLANET])
+		dmp->ncp = 0;
+	}
+
+	if (db_wantnodups)
+	    db_nodups();
+}
+
+/* called to indicate whether we want to eliminate dups.
+ * if turning it on, we go ahead and eliminate them right now and set a
+ *   flag so whenever new items are read dups are discarded then too.
+ * if turning it off, we just reset the local flag.
+ */
+void
+db_setnodups (whether)
+int whether;
+{
+	if (whether) {
+	    db_nodups();
+	    all_newdb(0);
+	}
+
+	db_wantnodups = whether;
+}
+
 /* read the given database file into memory.
  * add to the existing list.
  * stop gracefully if we run out of memory.
  * return 0 if all ok, else -1.
  */
+int
 db_read (fp)
 FILE *fp;
 {
 	char buf[MAXDBLINE];
+	long len;
+	int ret;
 
 	if (!DBINITED)
 	    db_init();
 
+	/* set up to run the progress meter based on file position */
+	(void) fseek (fp, 0L, 2);
+	len = ftell (fp);
+	(void) fseek (fp, 0L, 0);
+	pm_set (0);
+
 	/* read each line from the file and add to the db */
-	while (nxt_db (buf, sizeof(buf), fp) == 0)
-	    if (db_crack_line (buf) < 0)
-		return (-1);
-	return (0);
+	ret = 0;
+	while (nxt_db (buf, sizeof(buf), fp) == 0) {
+	    pm_set ((int)(ftell(fp)*100/len)); /* update progress meter */
+	    if (db_crack_line (buf) < 0) {
+		ret = -1;
+		break;
+	    }
+	}
+
+	/* discard any duplicates */
+	if (db_wantnodups)
+	    db_nodups();
+
+	/* should have hit eof */
+	if (ret == 0)
+	    ret = feof(fp) ? 0 : -1;
+
+	return (ret);
 }
 
 /* assuming we can open it ok, connect the db fifo to a callback.
+ * we close and reopen each time we are called.
  */
 void
 db_connect_fifo()
 {
 	static char dbfifores[] = "DBFIFO";	/* X resource naming db fifo */
 	char *fn;
-	int fd;
 
+	/* get the fifo name from the X resource */
 	fn = XGetDefault (XtD, myclass, dbfifores);
 	if (!fn) {
 	    char msg[256];
@@ -360,31 +512,43 @@ db_connect_fifo()
 	    return;
 	}
 
+	/* close if currently open */
+	if (db_fifofd >= 0) {
+	    XtRemoveInput (db_fifoid);
+	    (void) close (db_fifofd);
+	    db_fifofd = -1;
+	}
+
 	/* open for read/write. this assures open will never block, that
 	 * reads (and hence select()) WILL block if it's empty, and let's
 	 * processes using it come and go as they please.
 	 */
-	fd = openh (fn, 2);
-	if (fd < 0) {
+	db_fifofd = openh (fn, 2);
+	if (db_fifofd < 0) {
 	    char msg[256];
 	    (void) sprintf (msg, "Can not open %.150s: %.50s\n",fn,syserrstr());
 	    xe_msg (msg, 0);
 	    return;
 	}
 
-	XtAppAddInput (xe_app, fd, (XtPointer)XtInputReadMask, dbfifo_cb,
-								(XtPointer)fn);
+	/* wait for messages */
+	db_fifoid = XtAppAddInput(xe_app, db_fifofd, (XtPointer)XtInputReadMask,
+						    dbfifo_cb, (XtPointer)fn);
 }
 
 /* given a text buffer and a field id, and a PREF_DATE_FORMAT,
  *   set the corresponding member in *op.
  * return 0 if ok, else -1.
  */
+int
 db_set_field (bp, id, pref, op)
 char bp[];
+int id;
 PrefDateFormat pref;
 Obj *op;
 {
+	double tmp;
+
 	/* include all the enums and in numeric order to give us the best
 	 * possible chance the compiler will implement this as a jump table.
 	 */
@@ -393,38 +557,23 @@ Obj *op;
 	    printf ("db_set_field: called with id==O_TYPE\n");
 	    exit(1);
 	    break;
-	case O_NAME: {
+	case O_NAME:
 	    (void) strncpy (op->o_name, bp, sizeof(op->o_name)-1);
 	    op->o_name[sizeof(op->o_name)-1] = '\0';
 	    break;
-	}
-	case F_RA: {
-	    int h, m, s;
-	    double f_ra;
-	    f_dec_sexsign (radhr(op->f_RA), &h, &m, &s);
-	    f_sscansex (bp, &h, &m, &s);
-	    f_ra = op->f_RA;
-	    sex_dec (h, m, s, &f_ra);
-	    op->f_RA = hrrad(f_ra);
+	case F_RA:
+	    f_scansex (radhr(op->f_RA), bp, &tmp);
+	    op->f_RA = hrrad(tmp);
 	    break;
-	}
-	case F_DEC: {
-	    int dg, m, s;
-	    double tdec;
-	    f_dec_sexsign (raddeg(op->f_dec), &dg, &m, &s);
-	    f_sscansex (bp, &dg, &m, &s);
-	    tdec = op->f_dec;
-	    sex_dec (dg, m, s, &tdec);
-	    op->f_dec = degrad(tdec);
+	case F_DEC:
+	    f_scansex (raddeg(op->f_dec), bp, &tmp);
+	    op->f_dec = degrad(tmp);
 	    break;
-	}
-	case F_EPOCH: {
-	    double fepoch;
-	    fepoch = op->f_epoch;
-	    crack_year (bp, pref, &fepoch);
-	    op->f_epoch = fepoch;
+	case F_EPOCH:
+	    tmp = op->f_epoch;
+	    crack_year (bp, pref, &tmp);
+	    op->f_epoch = tmp;
 	    break;
-	}
 	case F_MAG:
 	    op->f_mag = atod (bp) * MAGSCALE;
 	    break;
@@ -434,8 +583,9 @@ Obj *op;
 	case F_CLASS:
 	    switch (bp[0]) {
 	    case 'A': case 'B': case 'C': case 'D': case 'F': case 'G':
-	    case 'H': case 'K': case 'M': case 'N': case 'O': case 'P':
-	    case 'Q': case 'S': case 'T': case 'U': case 'V':
+	    case 'H': case 'K': case 'J': case 'L': case 'M': case 'N':
+	    case 'O': case 'P': case 'Q': case 'R': case 'S': case 'T':
+	    case 'U': case 'V':
 		op->f_class = bp[0];
 		break;
 	    default:
@@ -614,7 +764,7 @@ Obj *op;
  */
 static int
 db_crack_line (s)
-char *s;
+char s[];
 {
 	static char nomem[] = "Insufficient memory to load this database\n";
 #define	MAXFLDS	20		/* must be more than on any expected line */
@@ -642,17 +792,17 @@ char *s;
 	/* switch out on type of object - the second field */
 	switch (flds[1][0]) {
 	case 'f': {
-	    static int ids[] = {F_RA, F_DEC, F_MAG, F_EPOCH};
-	    if (nf != 6 && nf != 7) {
+	    static int ids[] = {F_RA, F_DEC, F_MAG};
+	    if (nf < 5 || nf > 7) {
 		(void)sprintf(msg,
-		    "Need ra,dec,mag,[epoch][,siz] for fixed object `%.460s'\n",
+		    "Need ra,dec,mag[,epoch][,siz] for fixed object `%.460s'\n",
 								    flds[0]);
 		xe_msg (msg, 0);
 		return (-1);
 	    }
 	    op = db_new(FIXED);
 	    if (!op) {
-		xe_msg (nomem, 1);
+		xe_msg (nomem, 0);
 		return (-1);
 	    }
 	    op->type = FIXED;
@@ -667,6 +817,8 @@ char *s;
 		(void) db_set_field (sflds[2], F_SPECT, PREF_MDY, op);
 	    for (i = 2; i < ASIZ(ids)+2; i++)
 		(void) db_set_field (flds[i], ids[i-2], PREF_MDY, op);
+	    (void) db_set_field (nf>5 && flds[5][0] ? flds[5] : "2000",
+							F_EPOCH, PREF_MDY, op);
 	    if (nf == 7)
 		(void) db_set_field (flds[6], F_SIZE, PREF_MDY, op);
 	    break;
@@ -685,7 +837,7 @@ char *s;
 	    }
 	    op = db_new(ELLIPTICAL);
 	    if (!op) {
-		xe_msg (nomem, 1);
+		xe_msg (nomem, 0);
 		return (-1);
 	    }
 	    op->type = ELLIPTICAL;
@@ -707,7 +859,7 @@ char *s;
 	    }
 	    op = db_new(HYPERBOLIC);
 	    if (!op) {
-		xe_msg (nomem, 1);
+		xe_msg (nomem, 0);
 		return (-1);
 	    }
 	    op->type = HYPERBOLIC;
@@ -729,7 +881,7 @@ char *s;
 	    }
 	    op = db_new(PARABOLIC);
 	    if (!op) {
-		xe_msg (nomem, 1);
+		xe_msg (nomem, 0);
 		return (-1);
 	    }
 	    op->type = PARABOLIC;
@@ -752,7 +904,7 @@ char *s;
 	    }
 	    op = db_new(EARTHSAT);
 	    if (!op) {
-		xe_msg (nomem, 1);
+		xe_msg (nomem, 0);
 		return (-1);
 	    }
 	    op->type = EARTHSAT;
@@ -811,7 +963,7 @@ int t;
 
 	op = OBJP (t, dmp->nobj);
 	dmp->nobj++;
-	zero_mem ((char *)op, dmp->size);
+	zero_mem ((void *)op, dmp->size);
 	return (op);
 }
 
@@ -856,6 +1008,137 @@ db_init()
 	}
 }
 
+/* save the type of object being sorted (for use by db_nodups()) */
+static int dup_t;
+
+/* qsort name comparison function (for use by db_nodups()) */
+static int
+dup_namecmpf (v1, v2)
+#ifdef __STDC__
+const
+#endif
+void *v1, *v2;
+{
+	int n1 = *((int *)v1);
+	int n2 = *((int *)v2);
+	Obj *op1 = OBJP (dup_t, n1);
+	Obj *op2 = OBJP (dup_t, n2);
+
+	return (strcmp (op1->o_name, op2->o_name));
+}
+
+/* qsort n comparison function (for use by db_nodups()) */
+static int
+dup_ncmpf (v1, v2)
+#ifdef __STDC__
+const
+#endif
+void *v1, *v2;
+{
+	int n1 = *((int *)v1);
+	int n2 = *((int *)v2);
+
+	return (n1 - n2);
+}
+
+/* eliminate all db entries with duplicate names in each type of object */
+static void
+db_nodups()
+{
+	int t;
+
+	watch_cursor (1);
+
+	for (t = 0; t < NOBJTYPES; t++) {
+	    DBMem *dbm;
+	    Obj *op1, *op2;
+	    int *idx;
+	    int n, s;
+	    int i, j;
+	    int nxtidx;
+	    int ndups;
+
+	    /* skip undefined objects (of course) and leave the predefined
+	     * objects alone.
+	     */
+	    if (t == UNDEFOBJ || t == PLANET)
+		continue;
+
+	    /* handy references */
+	    dbm = &db[t];
+	    n = dbm->nobj;
+	    s = dbm->size;
+
+	    /* build a list of indices sorted by name */
+	    idx = (int *) malloc (n * sizeof(int));
+	    if (!idx) {
+		xe_msg ("No memory for dup temp array.", 0);
+		continue;
+	    }
+	    for (i = 0; i < n; i++)
+		idx[i] = i;
+	    dup_t = t;
+	    qsort ((void *)idx, n, sizeof(int), dup_namecmpf);
+
+	    /* throw out all indeces which only occur once *in place*.
+	     * this will leave us with ndups at the front of idx.
+	     */
+	    for (i = 1, ndups = 0; i < n; i++) {
+		op1 = OBJP(t,idx[i]);
+		op2 = OBJP(t,idx[i-1]);
+		if (strcmp (op1->o_name, op2->o_name) == 0)
+		    idx[ndups++] = idx[i];
+	    }
+
+	    if (ndups == 0)
+		goto done;	/* no dups */
+
+	    /* sort the dup list in ascending order for faster checking */
+	    qsort ((void *)idx, ndups, sizeof(int), dup_ncmpf);
+
+	    /* now compress each of the ndups entries in idx out of the real db.
+	     * don't discard anything prior to the last checkpoint.
+	     * consider starting at idx[0] since it's the first dup to go.
+	     * exit loop with j being the new net number of objects.
+	     */
+	    nxtidx = 0;
+	    for (i = j = idx[0] > dbm->ncp ? idx[0] : dbm->ncp; i < n; i++) {
+		int in, k;
+
+		/* see if i is in idx[].
+		 * nxtidx and testing idx[k] against i keeps a bound on the
+		 * portion of idx which might contain i.
+		 */
+		in = 0;
+		for (k = nxtidx; k < ndups && idx[k] <= i; k++)
+		    if (idx[k] == i) {
+			in = 1;
+			nxtidx = k+1;
+			break;
+		    }
+
+		/* if i is in idx[] then it's a dup and we don't copy it */
+		if (!in) {
+		    /* copy i to j (avoid a copy onto itself) */
+		    if (i > j) {
+			op1 = OBJP(t,i);
+			op2 = OBJP(t,j);
+			(void) memcpy ((void *)op2, (void *)op1, s);
+		    }
+		    j++;
+		}
+	    }
+
+	    dbm->nobj = j;
+
+	done:
+	    free ((char *)idx);
+		
+	}
+
+	watch_cursor (0);
+}
+
 /* read database file fp and put next valid entry (sans trailing \n) into buf.
  * we only count those lines that begin with alpha or numeric chars.
  * return 0 if ok.
@@ -875,7 +1158,7 @@ FILE *fp;
 		return (-1);
 	    l = strlen(buf);
 	    if (buf[l-1] != '\n') {
-		xe_msg ("Databse file line length is too long\n", 1);
+		xe_msg("Database file line length is too long or corrupted.",0);
 		return (-1);
 	    }
 	    if (line_candidate (buf) == 0) {
@@ -920,10 +1203,10 @@ double *p;
  * N.B. 0-length fields count, so even if *s=='\0' we return 1.
  * return the number of fields.
  */
-static
+static int
 get_fields (s, delim, fields)
 char *s;
-char delim;
+int delim;
 char *fields[];
 {
 	int n;
@@ -961,9 +1244,8 @@ XtInputId *idp;         /* pointer to input id */
 	static int npartial;		/* length of stuff in partial[] */
 	char buf[16*1024];		/* nice big read gulps */
 	char *name = (char *)client;	/* fifo filename */
-	int fd = *fdp;			/* fifo file descriptor */
+	char msg[1024];			/* error message buffer */
 	int nr;				/* number of bytes read from fifo */
-	char msg[256];			/* error message buffer */
 
 	/* turn on the watch cursor if there's no prior line */
 	if (!npartial)
@@ -976,7 +1258,7 @@ XtInputId *idp;         /* pointer to input id */
 	/* read what's available up to the room we have left.
 	 * if we have no room left, it will look like an EOF.
 	 */
-	nr = read (fd, buf+npartial, sizeof(buf)-npartial);
+	nr = read (db_fifofd, buf+npartial, sizeof(buf)-npartial);
 
 	if (nr > 0) {
 	    char c, *lp, *bp, *ep;	/* last line, current, end */
@@ -986,9 +1268,13 @@ XtInputId *idp;         /* pointer to input id */
 	    for (lp = bp = buf; bp < ep; ) {
 		c = *bp++;
 		if (c == '\n') {
-		    if (line_candidate (lp) == 0) {
-			bp[-1] = '\0';		      /* replace nl with EOS */
-			(void) db_crack_line (lp);
+		    bp[-1] = '\0';		      /* replace nl with EOS */
+		    if (*lp == DBFIFO_MSG) {
+			(void)sprintf(msg,"DBFIFO message: %s", lp+1);
+			xe_msg (msg, 0);
+		    } else if (line_candidate (lp) == 0) {
+			if (db_crack_line (lp) < 0)
+			    xe_msg ("Error in data format from fifo stream", 0);
 		    }
 		    lp = bp;
 		}
@@ -1013,17 +1299,19 @@ XtInputId *idp;         /* pointer to input id */
 							name, syserrstr());
 	    else 
 		(void) sprintf (msg, "Unexpected EOF on %.200s.\n", name);
-	    xe_msg (msg, 0);
-	    XtRemoveInput (*idp);
-	    (void) close (fd);
+	    xe_msg (msg, 1);
+	    XtRemoveInput (db_fifoid);
+	    (void) close (db_fifofd);
+	    db_fifofd = -1;
 	    npartial = 0;
 	}
 
-	/* turn off the watch cursor if there is not likely to be more
-	 * coming.
-	 * Also, inform everyone about the next stuff.
+	/* turn off the watch cursor if there is not likely to be more coming.
+	 * Also, throw out dups and inform everyone about all the new stuff.
 	 */
 	if (!npartial) {
+	    if (db_wantnodups)
+		db_nodups();
 	    all_newdb (1);
 	    watch_cursor (0);
 	}
