@@ -19,84 +19,72 @@
 #include <Xm/RowColumn.h>
 #include <Xm/Separator.h>
 #include <Xm/SelectioB.h>
+
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 #include "preferences.h"
 
+
+extern Widget toplevel_w;
+extern char *myclass;
+extern XtAppContext xe_app;
+#define	XtD XtDisplay(toplevel_w)
+
+extern Obj *db_basic P_((int id));
+extern Widget calm_create P_((Widget parent));
+extern double atod P_((char *buf));
+extern int listing_ison P_((void));
+extern int plot_ison P_((void));
+extern int srch_eval P_((double Mjd, double *tmincp));
+extern int tz_fromsys P_((Now *np));
+extern void all_update P_((Now *np, int how_much));
+extern void calm_set P_((Now *np));
+extern void db_invalidate P_((void));
+extern void db_loadinitial P_((void));
+extern void dm_riset P_((Now *np, Obj *op, RiseSet *rsp));
+extern void f_date P_((Widget w, double jd));
+extern void f_dms_angle P_((Widget w, double a));
+extern void f_double P_((Widget w, char *fmt, double f));
+extern void f_mtime P_((Widget w, double t));
+extern void f_off P_((void));
+extern void f_on P_((void));
+extern void f_scansex P_((double o, char bp[], double *np));
+extern void f_sexa P_((Widget wid, double a, int w, int fracbase));
+extern void f_showit P_((Widget w, char *s));
+extern void f_sscandate P_((char *bp, int pref, int *m, double *d, int *y));
+extern void f_string P_((Widget w, char *s));
+extern void f_time P_((Widget w, double t));
+extern void get_something P_((Widget w, char *resource, XtArgVal value));
+extern void get_xmstring P_((Widget w, char *resource, char **txtp));
+extern void inc_mjd P_((Now *np, double inc, int rtcflag));
+extern void listing P_((void));
+extern void mm_reset P_((void));
+extern void moonnf P_((double Mjd, double *Mjdn, double *Mjdf));
+extern void now_lst P_((Now *np, double *lst));
+extern void plot P_((void));
+extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
+extern void redraw_screen P_((int how_much));
+extern void register_selection P_((char *name));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
+extern void set_t0 P_((Now *np));
+extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void time_fromsys P_((Now *np));
+extern void twilight_cir P_((Now *np, double dis, double *dawn, double *dusk,
+    int *status));
+extern void watch_cursor P_((int want));
+extern void xe_msg P_((char *msg, int app_modal));
+
 typedef struct {
     int id;		/* see below -- used just as a cross-check */
     char *prompt;	/* used when asking for new value or NULL if can't */
     char *label;	/* used for the menu label or NULL */
     char *name;		/* used when selecting for plotting or NULL if can't */
-    char *altp[3];	/* alternate prompts based on preferences */
+    char *altp[3];	/* alternate prompts */
     Widget pb_w;	/* pushbutton for display and changing/selecting */
 } Field;
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
 
-extern Obj *db_basic P_((int id));
-extern double atod P_((char *buf));
-extern double mjd_day P_((double jd));
-extern double mjd_hr P_((double jd));
-extern void moonnf P_((double Mjd, double *mjdn, double *mjdf));
-extern int f_ison P_((void));
-extern int listing_ison P_((void));
-extern int plot_cartesian P_((FILE *pfp, Widget widget, unsigned int nx,
-    unsigned int ny, int flipx, int flipy, int grid));
-extern int plot_ison P_((void));
-extern int srch_eval P_((double Mjd, double *tmincp));
-extern void all_update P_((Now *np, int how_much));
-extern void cal_mjd P_((int mn, double dy, int yr, double *Mjd));
-extern void calm_set P_((Now *np));
-extern void db_invalidate P_((void));
-extern void dm_riset P_((Now *np, Obj *op, RiseSet *rsp));
-extern void f_date P_((Widget w, double jd));
-extern void f_dec_sexsign P_((double x, int *h, int *m, int *s));
-extern void f_double P_((Widget w, char *fmt, double f));
-extern void f_dms_angle P_((Widget w, double a));
-extern void f_mtime P_((Widget w, double t));
-extern void f_off P_((void));
-extern void f_on P_((void));
-extern void f_signtime P_((Widget w, double t));
-extern void f_showit P_((Widget w, char *s));
-extern void f_sscandate P_((char *bp, int pref, int *m, double *d, int *y));
-extern void f_sscansex P_((char *bp, int *d, int *m, int *s));
-extern void f_string P_((Widget w, char *s));
-extern void f_time P_((Widget w, double t));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
-extern void gst_utc P_((double Mjd, double gst, double *utc));
-extern void inc_mjd P_((Now *np, double inc, int rtcflag));
-extern void listing P_((void));
-extern void mjd_cal P_((double Mjd, int *mn, double *dy, int *yr));
-extern void mjd_year P_((double Mjd, double *yr));
-extern void now_lst P_((Now *np, double *lst));
-extern void plot P_((void));
-extern void plot_manage P_((void));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void range P_((double *v, double r));
-extern void register_selection P_((char *name));
-extern void set_t0 P_((Now *np));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void set_something P_((Widget w, char *resource, char *value));
-extern void get_something P_((Widget w, char *resource, char *value));
-extern void sex_dec P_((int hd, int m, int s, double *d));
-extern void time_fromsys P_((Now *np));
-extern void twilight_cir P_((Now *np, double dis, double *dawn, double *dusk,
-    int *status));
-extern void watch_cursor P_((int want));
-extern void xe_msg P_((char *msg, int app_modal));
-extern void year_mjd P_((double y, double *Mjd));
-
-void mm_manage P_((Widget main_window));
-void mm_reset P_((void));
-void mm_newcaldate P_((double newmjd));
-void mm_selection_mode P_((int whether));
-Now *mm_get_now P_((void));
-void redraw_screen P_((int how_much));
 static void create_main_form P_((Widget main_window));
 static void read_default_resources P_((void));
 static Widget fw P_((int fid));
@@ -104,7 +92,7 @@ static void mm_new_now P_((void));
 static void mm_set_step_code P_((char *bp));
 static void mm_set_buttons P_((int whether));
 static void mm_activate_cb P_((Widget w, XtPointer client, XtPointer call));
-static void mm_set_pref_prompts P_((Field *fp));
+static void mm_set_alt_prompts P_((Field *fp));
 static void mm_timer_cb P_((XtPointer client, XtIntervalId *id));
 static void mm_go_cb P_((Widget w, XtPointer client, XtPointer call));
 static void mm_step_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -125,14 +113,6 @@ static void mm_twilight P_((int force));
 static void mm_newcir P_((int y));
 static void mm_newcir_cb P_((XtPointer client, XtIntervalId *id));
 
-#undef P_
-
-extern Widget toplevel_w;
-extern char *myclass;
-extern XtAppContext xe_app;
-#define	XtD XtDisplay(toplevel_w)
-extern Widget calm_create();
-
 /* shorthands for fields of a Now structure, now.
  * first undo the ones for a Now pointer from circum.h.
  */
@@ -163,13 +143,22 @@ extern Widget calm_create();
 static Now now = {XMJD};/* where when and how, right now */
 static double tminc;	/* hrs to inc time by each loop, but see StepOptions */
 static int nstep;	/* steps to go before stopping */
-static int spause;	/* secs to pause between steps */
-static int newcir = 1; /* set when circumstances change - means don't tminc */
+static int spause;	/* msecs to pause between steps */
+static int newcir = 1;	/* set when circumstances change - means don't tminc */
+static int movie;	/* true while a movie is running */
+static int autotz = 0;	/* use tz_fromsys() each time step if this is on.
+			 * set by using any Now; reset if explicitly set TZ Name
+			 * or Offset.
+			 */
+
+/* strings used to command and prompt for turning autotz on and off */
+static char AutoOn[] = "Auto on";
+static char AutoOff[] = "Auto off";
 
 static XtIntervalId mm_interval_id;	/* set while waiting in a pause loop */
 static int mm_selecting;        /* set while our fields are being selected */
 
-#define	NO_PAUSE_DELAY	20	/* ms between loops when pause is 0 */
+#define	NO_PAUSE_DELAY	50	/* min ms delay to use, regardless of spause */
 #define	NC_BLINK_DELAY	200	/* ms between NEW CIRCUMSTANCES state changes */
 
 /* field ids
@@ -184,9 +173,9 @@ enum {
 /* array of label/button pairs.
  * N.B. these must be in the same order as the _FID enums so the XX_FID may
  *   be used as indices into the array.
- * N.B. some of the prompts get set based on preferences but we need to put
- *   *something* in the prompt field of selectable fields to trigger making a
- *   button. These alternate prompts must be set to match the PREF_ enums.
+ * N.B. some of the prompts get set based on preferences or other criteria but
+ *   we need to put *something* in the prompt field of selectable fields to
+ *   trigger making a button. See mm_set_alt_prompts().
  */
 static Field mm_field_map[] = {
     {UD_FID,	"dummy",				"UTC Date:","UD",
@@ -198,8 +187,22 @@ static Field mm_field_map[] = {
     {UT_FID,	"UTC time (h:m:s): ",			"UTC Time:", "UT"},
     {JD_FID,	"Julian Date: ",			"Julian:", "JD"},
     {LST_FID,	"Local sidereal time (h:m:s): ",	"Sidereal:", "LST"},
-    {TZN_FID,	"Timezone abbreviation (4 char max): ",	"TZ Name:", NULL},
-    {TZONE_FID,	"Hours behind UTC: ",			"TZ Offset:", "TZ"},
+    {TZN_FID,	"dummy",				"TZ Name:", NULL,
+	{    "Enter Timezone abbreviation (7 char max),\n\
+or select Auto On to track OS: ",
+	     "Enter Timezone abbreviation (7 char max),\n\
+or select Auto Off to turn off tracking: ",
+	     NULL
+	}
+    },
+    {TZONE_FID,	"dummy",				"TZ Offset:", "TZ",
+	{    "Enter Hours behind UTC (h:m:s),\n\
+or select Auto On to track OS: ",
+	     "Enter Hours behind UTC (h:m:s),\n\
+or select Auto Off to turn off tracking: ",
+	     NULL
+	}
+    },
     {LD_FID,	"dummy",				"Local Date:", "LD",
 	{   "Local date (m/d/y or year.d): ",
 	    "Local date (y/m/d or year.d): ",
@@ -253,24 +256,24 @@ static Widget go_w;
 /* N.B. index of item is its switch/case value in crack_fieldset ()
  */
 static char *keywords[] = {
-    /*  0 */	"Epoch",
-    /*  1 */	"Elevation",
-    /*  2 */	"JD",
-    /*  3 */	"Lat",
-    /*  4 */	"Long",
-    /*  5 */	"NSteps",
-    /*  6 */	"Pause",
-    /*  7 */	"Pressure",
-    /*  8 */	"StepSize",
-    /*  9 */	"Temp",
-    /* 10 */	"UD",
-    /* 11 */	"UT",
-    /* 12 */	"TwilightDip",
-    /* 13 */	"LD",
-    /* 14 */	"LT",
-    /* 15 */	"LST",
-    /* 16 */	"TZName",	/* these should be last */
-    /* 17 */	"TZone",
+    /*  0 */	"TZName",	/* these should be first so others may use */
+    /*  1 */	"TZone",
+    /*  2 */	"Epoch",
+    /*  3 */	"Elevation",
+    /*  4 */	"JD",
+    /*  5 */	"Lat",
+    /*  6 */	"Long",
+    /*  7 */	"NSteps",
+    /*  8 */	"Pause",
+    /*  9 */	"Pressure",
+    /* 10 */	"StepSize",
+    /* 11 */	"Temp",
+    /* 12 */	"UD",
+    /* 13 */	"UT",
+    /* 14 */	"TwilightDip",
+    /* 15 */	"LD",
+    /* 16 */	"LT",
+    /* 17 */	"LST",
 };
 
 /* stuff to manage the step control facility.
@@ -294,7 +297,7 @@ static StepOption step_options[] = {
     {"RT Clock",	RTC_SS},
     {"24:00:00",	FIXED_SS,	24.0},
     {"1:00:00",		FIXED_SS,	1.0},
-    {"0:01:00",		FIXED_SS,	1./60.},
+    {"0:05:00",		FIXED_SS,	5./60.},
     {"Next Dawn",	NEXTDAWN_SS},
     {"Next Dusk",	NEXTDUSK_SS},
     {"Next Sun rise",	NEXTSUNRISE_SS},
@@ -332,6 +335,9 @@ Widget main_window;
 	    (void) chg_fld ("Now", &mm_field_map[UD_FID]);
 	    redraw_screen (1);
 	}
+
+	/* load the initial set of objects.*/
+	db_loadinitial();
 }
 
 /* reset things to their initial resource settings */
@@ -353,6 +359,8 @@ double newmjd;
 {
 	mjd = newmjd;
 	set_t0 (&now);
+	if (autotz)
+	    (void) tz_fromsys (&now);
 	mm_now (1);
 	mm_newcir(1);
 	newcir = 1;
@@ -368,8 +376,8 @@ int whether;	/* whether setting up for plotting or for not plotting */
 {
 	mm_selecting += whether ? 1 : -1;
 
-	if (whether && mm_selecting == 1     /* first one to want on */
-	    || !whether && mm_selecting == 0 /* last one to want off */)
+	if ((whether && mm_selecting == 1)     /* first one to want on */
+	    || (!whether && mm_selecting == 0) /* last one to want off */)
 	    mm_set_buttons (whether);
 }
 
@@ -380,6 +388,51 @@ mm_get_now()
 	return (&now);
 }
 
+/* called to start or stop a movie sequence.
+ * if a movie is currently running, we stop it, period.
+ * then if stepsz != 0 we start one by set movie=1, spause=0, nstep=1000000,
+ *   tm_inc=stepsz and do about what the Go button does.
+ */
+void
+mm_movie (stepsz)
+double stepsz;	/* step size, hours */
+{
+	/* stop any current movie */
+	if (movie) {
+	    if (mm_interval_id != 0) {
+		XtRemoveTimeOut (mm_interval_id);
+		mm_interval_id = 0;
+	    }
+	    nstep = 1;
+	    redraw_screen (1);
+	    print_idle();
+	    movie = 0;
+	    return;
+	}
+
+	/* that's it if zero step size */
+	if (stepsz == 0.0)
+	    return;
+
+	/* note movie is about to be running */
+	movie = 1;
+
+	/* init nstep and spause to reasonable for long fast looping */
+	nstep = 1000000;
+	print_nstep(1);
+	spause = 0;
+	print_spause(1);
+
+	/* set the time increment to stepsz */
+	mm_step_option = NULL;	/* no special increment */
+	tminc = stepsz;
+	print_tminc(1);
+	print_running();
+
+	/* start fresh looping */
+	mm_go();
+}
+
 /* draw all the stuff on the managed menus.
  * if how_much == 0 then just update fields that need it;
  * if how_much == 1 then redraw all fields;
@@ -496,7 +549,7 @@ Widget main_window;
 		    n = 0;
 		    fp->pb_w = XmCreatePushButton (rc_w, "Gap", args, n);
 		    XtManageChild (fp->pb_w);
-		    set_something (fp->pb_w, XmNsensitive, False);
+		    set_something (fp->pb_w, XmNsensitive, (XtArgVal)False);
 		    set_xmstring (fp->pb_w, XmNlabelString, " ");
 		    continue;
 		}
@@ -694,16 +747,16 @@ XtPointer call;
 		register_selection (fp->name);
 	} else {
 	    if (fp->prompt) {
-		mm_set_pref_prompts (fp);
+		mm_set_alt_prompts (fp);
 		prompt (fp);
 	    }
 	}
 }
 
-/* set up those prompts that depend in preferences.
+/* set up those prompts that use the alternates.
  */
 static void
-mm_set_pref_prompts(fp)
+mm_set_alt_prompts(fp)
 Field *fp;
 {
 	switch (fp->id) {
@@ -716,6 +769,10 @@ Field *fp;
 	case PRES_FID:
 	    fp->prompt = fp->altp[pref_get(PREF_UNITS)];
 	    break;
+	case TZN_FID:
+	case TZONE_FID:
+	    fp->prompt = fp->altp[autotz];
+	    break;
 	}
 }
 
@@ -728,15 +785,7 @@ mm_timer_cb (client, id)
 XtPointer client;
 XtIntervalId *id;
 {
-	int waited_so_far = (int)client + 1;
-
-	mm_interval_id = 0;
-
-	if (waited_so_far < spause)
-	    mm_interval_id = XtAppAddTimeOut (xe_app, 1000, mm_timer_cb,
-						    (XtPointer)waited_so_far);
-	else
-	    mm_go();
+	mm_go();
 }
 
 /* callback from the main "go" button being armed.
@@ -754,6 +803,7 @@ XtPointer call;
 	    mm_interval_id = 0;
 	    redraw_screen (1);
 	    print_idle();
+	    movie = 0;	/* no movie running now either :-) */
 	} else {
 	    if (nstep > 1)
 		print_running();
@@ -834,9 +884,8 @@ mm_go()
 	    }
 	    print_idle();
 	} else {
-	    mm_interval_id =
-		XtAppAddTimeOut (xe_app, spause>0 ? 1000 : NO_PAUSE_DELAY,
-								mm_timer_cb, 0);
+	    int ms = spause < NO_PAUSE_DELAY ? NO_PAUSE_DELAY : spause;
+	    mm_interval_id = XtAppAddTimeOut (xe_app, ms, mm_timer_cb, 0);
 	}
 
 	XSync (XtD, False);
@@ -1010,6 +1059,9 @@ mm_new_now()
 	    printf ("mm_new_now(): bogus stepcode: %d\n", stepcode);
 	    exit (1);
 	}
+
+	if (autotz)
+	    (void) tz_fromsys (&now);
 }
 
 /* ARGSUSED */
@@ -1038,7 +1090,7 @@ int force;
 	    if (fabs(tminc) >= 24.0)
 		f_double (stpsz_w, "%6.4g dy", tminc/24.0);
 	    else
-		f_signtime (stpsz_w, tminc);
+		f_sexa (stpsz_w, tminc, 3, 3600);
 	    break;
 
 	default:
@@ -1075,7 +1127,7 @@ char *s;
 
 	if (s != last_s) {
 	    f_string (status_w, s);
-	    XFlush (XtD);
+	    XSync (XtD, False);
 	    last_s = s;
 	}
 }
@@ -1102,7 +1154,7 @@ int force;
 
 	if (force || spause != last) {
 	    char buf[16];
-	    (void) sprintf (buf, "%8d", spause);
+	    (void) sprintf (buf, "%8g", spause/1000.0);
 	    f_string (fw(PAUSE_FID), buf);
 	    last = spause;
 	}
@@ -1111,7 +1163,7 @@ int force;
 /* process a field spec in buf, either from config file or argv.
  * return 0 if recognized ok, else -1.
  */
-static
+static int
 crack_fieldset (buf)
 char *buf;
 {
@@ -1127,24 +1179,24 @@ char *buf;
 
 	/* N.B. the switch entries must match the order in keywords[] */
 	switch (i) {
-	case 0:  (void) chg_fld (buf, &mm_field_map[EPOCH_FID]); break;
-	case 1:  (void) chg_fld (buf, &mm_field_map[ELEV_FID]); break;
-	case 2:  (void) chg_fld (buf, &mm_field_map[JD_FID]); break;
-	case 3:  (void) chg_fld (buf, &mm_field_map[LAT_FID]); break;
-	case 4:  (void) chg_fld (buf, &mm_field_map[LONG_FID]); break;
-	case 5:  (void) chg_fld (buf, &mm_field_map[NSTEP_FID]); break;
-	case 6:  (void) chg_fld (buf, &mm_field_map[PAUSE_FID]); break;
-	case 7:  (void) chg_fld (buf, &mm_field_map[PRES_FID]); break;
-	case 8:  (void) chg_fld (buf, &mm_field_map[STPSZ_FID]); break;
-	case 9:  (void) chg_fld (buf, &mm_field_map[TEMP_FID]); break;
-	case 10: (void) chg_fld (buf, &mm_field_map[UD_FID]); break;
-	case 11: (void) chg_fld (buf, &mm_field_map[UT_FID]); break;
-	case 12: (void) chg_fld (buf, &mm_field_map[DIP_FID]); break;
-	case 13: (void) chg_fld (buf, &mm_field_map[LD_FID]); break;
-	case 14: (void) chg_fld (buf, &mm_field_map[LT_FID]); break;
-	case 15: (void) chg_fld (buf, &mm_field_map[LST_FID]); break;
-	case 16: (void) chg_fld (buf, &mm_field_map[TZN_FID]); break;
-	case 17: (void) chg_fld (buf, &mm_field_map[TZONE_FID]); break;
+	case 0:  (void) chg_fld (buf, &mm_field_map[TZN_FID]); break;
+	case 1:  (void) chg_fld (buf, &mm_field_map[TZONE_FID]); break;
+	case 2:  (void) chg_fld (buf, &mm_field_map[EPOCH_FID]); break;
+	case 3:  (void) chg_fld (buf, &mm_field_map[ELEV_FID]); break;
+	case 4:  (void) chg_fld (buf, &mm_field_map[JD_FID]); break;
+	case 5:  (void) chg_fld (buf, &mm_field_map[LAT_FID]); break;
+	case 6:  (void) chg_fld (buf, &mm_field_map[LONG_FID]); break;
+	case 7:  (void) chg_fld (buf, &mm_field_map[NSTEP_FID]); break;
+	case 8:  (void) chg_fld (buf, &mm_field_map[PAUSE_FID]); break;
+	case 9:  (void) chg_fld (buf, &mm_field_map[PRES_FID]); break;
+	case 10: (void) chg_fld (buf, &mm_field_map[STPSZ_FID]); break;
+	case 11: (void) chg_fld (buf, &mm_field_map[TEMP_FID]); break;
+	case 12: (void) chg_fld (buf, &mm_field_map[UD_FID]); break;
+	case 13: (void) chg_fld (buf, &mm_field_map[UT_FID]); break;
+	case 14: (void) chg_fld (buf, &mm_field_map[DIP_FID]); break;
+	case 15: (void) chg_fld (buf, &mm_field_map[LD_FID]); break;
+	case 16: (void) chg_fld (buf, &mm_field_map[LT_FID]); break;
+	case 17: (void) chg_fld (buf, &mm_field_map[LST_FID]); break;
 	default: return (-1);
 	}
 	return (0);
@@ -1156,28 +1208,29 @@ char *buf;
  * return 1 if we change a field that invalidates any of the times or
  * to update all related fields.
  */
-static
+static int
 chg_fld (bp, fp)
 char *bp;
 Field *fp;
 {
-	int deghrs = 0, mins = 0, secs = 0;
 	int new = 0;
 	double tmp;
 
 	switch (fp->id) {
 	case JD_FID:
-	    if (bp[0] == 'n' || bp[0] == 'N')
+	    if (bp[0] == 'n' || bp[0] == 'N') {
 		time_fromsys (&now);
-	    else
+		autotz = 1;
+	    } else
 		mjd = atod(bp) - MJD0;
 	    set_t0 (&now);
 	    new = 1;
 	    break;
 	case UD_FID:
-	    if (bp[0] == 'n' || bp[0] == 'N')
+	    if (bp[0] == 'n' || bp[0] == 'N') {
 		time_fromsys (&now);
-	    else {
+		autotz = 1;
+	    } else {
 		double day, newmjd0;
 		int month, year;
 
@@ -1198,22 +1251,23 @@ Field *fp;
 	    new = 1;
 	    break;
 	case UT_FID:
-	    if (bp[0] == 'n' || bp[0] == 'N')
+	    if (bp[0] == 'n' || bp[0] == 'N') {
 		time_fromsys (&now);
-	    else {
+		autotz = 1;
+	    } else {
 		double newutc = (mjd-mjd_day(mjd)) * 24.0;
-		f_dec_sexsign (newutc, &deghrs, &mins, &secs);
-		f_sscansex (bp, &deghrs, &mins, &secs);
-		sex_dec (deghrs, mins, secs, &newutc);
+		rnd_second (&newutc);	/* match display precision */
+		f_scansex (newutc, bp, &newutc);
 		mjd = mjd_day(mjd) + newutc/24.0;
 	    }
 	    set_t0 (&now);
 	    new = 1;
 	    break;
 	case LD_FID:
-	    if (bp[0] == 'n' || bp[0] == 'N')
+	    if (bp[0] == 'n' || bp[0] == 'N') {
 		time_fromsys (&now);
-	    else {
+		autotz = 1;
+	    } else {
 		double day, newlmjd0;
 		int month, year;
 
@@ -1235,28 +1289,31 @@ Field *fp;
 	    new = 1;
 	    break;
 	case LT_FID:
-	    if (bp[0] == 'n' || bp[0] == 'N')
+	    if (bp[0] == 'n' || bp[0] == 'N') {
 		time_fromsys (&now);
-	    else {
+		autotz = 1;
+	    } else if (bp[0] == 'M' || bp[0] == 'm') {
+		/* midnight, tonight */
+		mjd = mjd_day(mjd+1-tz/24.0) + tz/24.0;
+	    } else {
 		double newlt = (mjd-mjd_day(mjd)) * 24.0 - tz;
 		range (&newlt, 24.0);
-		f_dec_sexsign (newlt, &deghrs, &mins, &secs);
-		f_sscansex (bp, &deghrs, &mins, &secs);
-		sex_dec (deghrs, mins, secs, &newlt);
+		rnd_second (&newlt);	/* match display precision */
+		f_scansex (newlt, bp, &newlt);
 		mjd = mjd_day(mjd-tz/24.0) + (newlt + tz)/24.0;
 	    }
 	    set_t0 (&now);
 	    new = 1;
 	    break;
 	case LST_FID:
-	    if (bp[0] == 'n' || bp[0] == 'N')
+	    if (bp[0] == 'n' || bp[0] == 'N') {
 		time_fromsys (&now);
-	    else {
+		autotz = 1;
+	    } else {
 		double lst, utc;
 		now_lst (&now, &lst);
-		f_dec_sexsign (lst, &deghrs, &mins, &secs);
-		f_sscansex (bp, &deghrs, &mins, &secs);
-		sex_dec (deghrs, mins, secs, &lst);
+		rnd_second (&lst);	/* match display precision */
+		f_scansex (lst, bp, &lst);
 		lst -= radhr(lng); /* convert to gst */
 		range (&lst, 24.0);
 		gst_utc (mjd_day(mjd), lst, &utc);
@@ -1265,28 +1322,44 @@ Field *fp;
 	    set_t0 (&now);
 	    new = 1;
 	    break;
+
 	case TZN_FID:
-	    (void) strncpy (tznm, bp, sizeof(tznm)-1);
-	    new = 1;
-	    break;
 	case TZONE_FID:
-	    f_dec_sexsign (tz, &deghrs, &mins, &secs);
-	    f_sscansex (bp, &deghrs, &mins, &secs);
-	    sex_dec (deghrs, mins, secs, &tz);
-	    new = 1;
+	    if (strncmp (bp, AutoOn, strlen(AutoOn)) == 0) {
+		if (tz_fromsys (&now) < 0) {
+		    xe_msg ("Can not get timezone info from OS", 0);
+		    autotz = 0;
+		} else {
+		    autotz = 1;
+		    new = 1;
+		}
+	    } else if (strncmp (bp, AutoOff, strlen(AutoOff)) == 0) {
+		autotz = 0;
+	    } else {
+		if (fp->id == TZN_FID)
+		    (void) strncpy (tznm, bp, sizeof(tznm)-1);
+		else {
+		    /* TZONE_FID */
+		    rnd_second (&tz);	/* match display precision */
+		    f_scansex (tz, bp, &tz);
+		}
+		autotz = 0;
+		new = 1;
+	    }
 	    break;
+
 	case LONG_FID:
-	    f_dec_sexsign (-raddeg(lng), &deghrs, &mins, &secs);
-	    f_sscansex (bp, &deghrs, &mins, &secs);
-	    sex_dec (deghrs, mins, secs, &lng);
-	    lng = degrad (-lng); 		/* want - radians west */
+	    tmp = -raddeg(lng);
+	    rnd_second (&tmp);		/* match display precision */
+	    f_scansex (tmp, bp, &tmp);
+	    lng = degrad (-tmp); 	/* want <0 radians west */
 	    new = 1;
 	    break;
 	case LAT_FID:
-	    f_dec_sexsign (raddeg(lat), &deghrs, &mins, &secs);
-	    f_sscansex (bp, &deghrs, &mins, &secs);
-	    sex_dec (deghrs, mins, secs, &lat);
-	    lat = degrad (lat);
+	    tmp = raddeg(lat);
+	    rnd_second (&tmp);		/* match display precision */
+	    f_scansex (tmp, bp, &tmp);
+	    lat = degrad (tmp);
 	    new = 1;
 	    break;
 	case ELEV_FID:
@@ -1313,20 +1386,34 @@ Field *fp;
 	    print_nstep (0);
 	    break;
 	case PAUSE_FID:
-	    (void) sscanf (bp, "%d", &spause);
+	    (void) sscanf (bp, "%lf", &tmp);
+	    spause = tmp*1000 + 0.5;
 	    print_spause (0);
 	    break;
 	case TEMP_FID:
 	    if (sscanf (bp, "%lf", &temp) == 1) {
 		if (pref_get(PREF_UNITS) == PREF_ENGLISH)
 		    temp = 5./9.*(temp - 32.0);	/* want degs C */
+		if (temp < -100 || temp > 100) {
+		    char msg[128];
+		    (void) sprintf (msg,
+				"NOTICE: very unusual temperature: %g C", temp);
+		    xe_msg (msg, 0);
+		}
 		new = 1;
 	    }
 	    break;
 	case PRES_FID:
 	    if (sscanf (bp, "%lf", &pressure) == 1) {
-		if (pref_get(PREF_UNITS) == PREF_ENGLISH)
+		if (pref_get(PREF_UNITS) == PREF_ENGLISH) 
 		    pressure *= 33.86;		/* want mBar */
+		if (pressure < 0 || pressure > 4000) {
+		    char msg[128];
+		    (void) sprintf (msg,
+			    "NOTICE: very unusual atmospheric pressure: %g mB",
+								    pressure);
+		    xe_msg (msg, 0);
+		}
 		new = 1;
 	    }
 	    break;
@@ -1383,9 +1470,11 @@ XtPointer call;
 	    exit(1);
 	}
 
-	get_something (w, XmNuserData, (char *)&fp);
+	get_something (w, XmNuserData, (XtArgVal)&fp);
 
 	if (chg_fld (text, fp)) {
+	    if (autotz)
+		(void) tz_fromsys (&now);
 	    mm_now (1);
 	    mm_newcir(1);
 	    newcir = 1;
@@ -1476,15 +1565,27 @@ Field *fp;
 	set_xmstring(prompt_w, XmNtextString, "");
 
 	/* save fp in userData for the callbacks to get at */
-	set_something (prompt_w, XmNuserData, (char *)fp);
+	set_something (prompt_w, XmNuserData, (XtArgVal)fp);
 
 	/* set up for the special shortcuts */
 	switch (fp->id) {
+	case TZN_FID: case TZONE_FID:
+	    XtManageChild (aw);
+	    set_xmstring (prompt_w, XmNapplyLabelString,
+					    autotz ? AutoOff : AutoOn);
+	    break;
 	case JD_FID: case UD_FID:
-	case LD_FID: case LT_FID: case LST_FID:
+	case LD_FID: case LST_FID:
 	    XtManageChild (aw);
 	    set_xmstring (prompt_w, XmNapplyLabelString, "Now");
 	    break;
+	case LT_FID:
+	    XtManageChild (aw);
+	    if (mjd_hr(mjd) - tz == 0.0)
+		set_xmstring (prompt_w, XmNapplyLabelString, "Now");
+	    else
+		set_xmstring (prompt_w,XmNapplyLabelString,"Midnight\nTonight");
+	    break;
 	case UT_FID:
 	    XtManageChild (aw);
 	    if (mjd_hr(mjd) == 0.0)
@@ -1520,7 +1621,7 @@ Field *fp;
 	    if (pressure > 0) {
 		XtManageChild (aw);
 		set_xmstring (prompt_w, XmNapplyLabelString,
-							"0 (No refraction)");
+							"0\n(No refraction)");
 	    }
 	    break;
 	}
@@ -1574,13 +1675,9 @@ char *bp;
 		if (sscanf (bp, "%lf", &x) == 1)
 		    tminc = x * 24.0 * SIDRATE;
 	    } else {
-		int deghrs, mins, secs;
 		if (mm_step_option == RTC_SS_OPTION)
-		    deghrs = mins = secs = 0;	/* no prior */
-		else
-		    f_dec_sexsign (tminc, &deghrs, &mins, &secs); /* prior */
-		f_sscansex (bp, &deghrs, &mins, &secs);
-		sex_dec (deghrs, mins, secs, &tminc);
+		    tminc = 0;	/* no prior */
+		f_scansex (tminc, bp, &tminc);
 	    }
 	}
 }
@@ -1597,13 +1694,15 @@ int all;
 	double jd = mjd + MJD0;
 	double tmp;
 
-	(void) sprintf (buf, "%-*.*s", sizeof(tznm)-1, sizeof(tznm)-1, tznm);
+	(void) sprintf (buf, "%-*.*s", (int)sizeof(tznm)-1, (int)sizeof(tznm)-1,
+									tznm);
 	f_string (fw(TZN_FID), buf);
+	f_sexa (fw(TZONE_FID), tz, 3, 3600);
 	f_time (fw(LT_FID), mjd_hr(lmjd));
-	f_date (fw(LD_FID), lmjd);
+	f_date (fw(LD_FID), mjd_day(lmjd));
 
 	f_time (fw(UT_FID), mjd_hr(mjd));
-	f_date (fw(UD_FID), mjd);
+	f_date (fw(UD_FID), mjd_day(mjd));
 
 	f_double (fw(JD_FID), "%14.5f", jd);
 
@@ -1644,8 +1743,6 @@ int all;
 		f_double (fw(PRES_FID), "%5.0f mB", tmp);
 	    }
 
-	    f_signtime (fw(TZONE_FID), tz);
-
 	    if (epoch == EOD)
 		f_string (fw(EPOCH_FID), "Of Date");
 	    else {
@@ -1654,8 +1751,7 @@ int all;
 	    }
 	}
 
-	if (f_ison())
-	    calm_set (&now);
+	calm_set (&now);
 }
 
 /* display dawn/dusk/length-of-night times.
@@ -1670,6 +1766,15 @@ int force;
 	char buf[64];
 
 	twilight_cir (&now, dip, &dawn, &dusk, &status);
+
+	/* twilight_cir gives local times; switch to UTC if prefered */
+	if (pref_get(PREF_ZONE) == PREF_UTCTZ) {
+	    dawn += now.n_tz;
+	    range (&dawn, 24.0);
+	    dusk += now.n_tz;
+	    range (&dusk, 24.0);
+	}
+
 	if (status & (RS_NORISE|RS_RISERR|RS_NOSET|RS_CIRCUMPOLAR|RS_NEVERUP)) {
 	    static char nope[] = "-----";
 	    f_string (fw(DAWN_FID), nope);
