@@ -13,42 +13,38 @@
 #include <Xm/DrawingA.h>
 #include <Xm/Label.h>
 #include <Xm/PushB.h>
+#include <Xm/CascadeB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/Separator.h>
+
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 
 
 extern Widget toplevel_w;
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
-
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
-extern void llibration P_((double JD, double *llatp, double *llonp));
-extern void db_update P_((Obj *op));
-extern void get_something P_((Widget w, char *resource, char *value));
-extern void register_selection P_((char *name));
 extern int any_ison P_((void));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void f_dms_angle P_((Widget w, double a));
-extern void f_double P_((Widget w, char *fmt, double f));
 extern int get_color_resource P_((Widget w, char *cname, Pixel *p));
+extern void db_update P_((Obj *op));
+extern void f_double P_((Widget w, char *fmt, double f));
+extern void f_pangle P_((Widget w, double a));
+extern void get_something P_((Widget w, char *resource, XtArgVal value));
+extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
+extern void llibration P_((double JD, double *llatp, double *llonp));
+extern void register_selection P_((char *name));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
+extern void set_xmstring P_((Widget w, char *resource, char *txt));
 extern void timestamp P_((Now *np, Widget w));
 
-void m_manage P_((void));
-int m_ison P_((void));
-void m_update P_((Now *np, int how_much));
-void m_selection_mode P_((int whether));
-void m_cursor P_((Cursor c));
 static void m_create_form P_((void));
+static void m_create_msform P_((void));
 static void m_set_buttons P_((int whether));
 static void m_set_a_button P_((Widget pbw, int whether));
+static void m_mstats_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_activate_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_mapvw_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -62,8 +58,6 @@ static void m_draw_grview P_((double el, double limb));
 static void m_mark_libr P_((Display *dsp, Window win, GC gc, double limb,
     int rad, int tb, int lb));
 
-#undef P_
-
 /* get the small moonbit map and its dimension defines.
  * moon is roughly centered within the map, with a radius of some 200 pixels.
  * exact values set by trial and error.
@@ -77,14 +71,17 @@ static void m_mark_libr P_((Display *dsp, Window win, GC gc, double limb,
 #define	LIMBMARKFRAC	20	/* libr marker is 1/x this of total size */
 
 static Widget mform_w;		/* main moon form dialog */
+static Widget msform_w;		/* statistics form dialog */
 static Widget mda_w;		/* moon drawring area */
 static Widget eshine_w;		/* whether we want to show earthshine */
 static Widget mapvw_w;		/* toggle button that selects which view */
-static Widget dt_w;		/* date/time stamp widget */
+static Widget dt_w;		/* main date/time stamp widget */
+static Widget sdt_w;		/* statistics date/time stamp widget */
 static Widget llat_w;		/* lib in lat pushbutton widget */
 static Widget llong_w;		/* lib in long pushbutton widget */
 static Widget limb_w;		/* limb angle pushbutton widget */
 static Widget lib_w;		/* total lib pushbutton widget */
+static int mswasman;            /* whether statistics form was managed */
 
 #define	MAPVW_W	(smallfm_width+20)
 #define	MAPVW_H	(smallfm_height+40)
@@ -98,18 +95,27 @@ static int m_selecting;        /* set while our fields are being selected */
 void
 m_manage ()
 {
-	if (!mform_w)
+	if (!mform_w) {
 	    m_create_form();
+	    m_create_msform();
+	}
 	
-	if (XtIsManaged(mform_w))
+	if (XtIsManaged(mform_w)) {
 	    XtUnmanageChild (mform_w);
-	else {
+	    if ((mswasman = XtIsManaged(msform_w)))
+		XtUnmanageChild (msform_w);
+
+	} else {
 	    XtManageChild (mform_w);
-	    m_set_buttons(m_selecting);
+	    if (mswasman) {
+		XtManageChild (msform_w);
+		m_set_buttons(m_selecting);
+	    }
 	    /* let the initial expose do the drawing */
 	}
 }
 
+int
 m_ison()
 {
 	return (mform_w && XtIsManaged(mform_w));
@@ -139,8 +145,8 @@ int whether;	/* whether setting up for plotting or for not plotting */
 	m_selecting += whether ? 1 : -1;
 
 	if (m_ison()) {
-	    if (whether && m_selecting == 1     /* first one to want on */
-		|| !whether && m_selecting == 0 /* last one to want off */)
+	    if ((whether && m_selecting == 1)     /* first one to want on */
+		|| (!whether && m_selecting == 0) /* last one to want off */)
 		m_set_buttons (whether);
 	}
 }
@@ -159,14 +165,22 @@ Cursor c;
 	    else
 		XUndefineCursor (dsp, win);
 	}
+
+	if (msform_w && (win = XtWindow(msform_w))) {
+	    Display *dsp = XtDisplay(msform_w);
+	    if (c)
+		XDefineCursor (dsp, win, c);
+	    else
+		XUndefineCursor (dsp, win);
+	}
 }
 
 static void
 m_create_form()
 {
 	Widget frame_w;
-	Widget f_w, fr_w;
-	Widget sep_w, rc_w, w;
+	Widget mb_w, pd_w, cb_w;
+	Widget w;
 	XmString str;
 	Arg args[20];
 	int n;
@@ -178,184 +192,114 @@ m_create_form()
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	mform_w = XmCreateFormDialog (toplevel_w, "Moon", args, n);
+	XtAddCallback (mform_w, XmNhelpCallback, m_help_cb, 0);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
 	 */
 	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Moon View"); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Moon view"); n++;
 	XtSetValues (XtParent(mform_w), args, n);
 
-	/* make a form to hold the bottom controls */
+	/* create the menu bar across the top */
 
 	n = 0;
-	XtSetArg (args[n], XmNfractionBase, 4); n++;
-	XtSetArg (args[n], XmNhorizontalSpacing, 5); n++;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	f_w = XmCreateForm (mform_w, "MoonCtlForm", args, n);
-	XtManageChild (f_w);
+	mb_w = XmCreateMenuBar (mform_w, "MB", args, n);
+	XtManageChild (mb_w);
 
-	    /* make the close button */
+	/* make the Control pulldown */
+
+	n = 0;
+	pd_w = XmCreatePulldownMenu (mb_w, "ControlPD", args, n);
 
 	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 0); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 1); n++;
-	    w = XmCreatePushButton (f_w, "Close", args, n);
+	    XtSetArg (args[n], XmNsubMenuId, pd_w);  n++;
+	    XtSetArg (args[n], XmNmnemonic, 'C'); n++;
+	    cb_w = XmCreateCascadeButton (mb_w, "ControlCB", args, n);
+	    set_xmstring (cb_w, XmNlabelString, "Control");
+	    XtManageChild (cb_w);
+
+	    /* add the close button */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "Close", args, n);
 	    XtAddCallback (w, XmNactivateCallback, m_close_cb, 0);
 	    XtManageChild (w);
 
-	    /* make the earthshine toggle button in a frame */
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 1); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 2); n++;
-	    fr_w = XmCreateFrame (f_w, "EarthshineFr", args, n);
-	    XtManageChild (fr_w);
-
-		n = 0;
-		eshine_w = XmCreateToggleButton (fr_w, "Earthshine", args, n);
-		XtAddCallback (eshine_w, XmNvalueChangedCallback,m_eshine_cb,0);
-		XtManageChild (eshine_w);
-
-	    /* make the view option toggle button in a frame */
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 2); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 3); n++;
-	    fr_w = XmCreateFrame(f_w,"MMapViewFr", args, n);
-	    XtManageChild (fr_w);
-
-		str = XmStringCreate("Bitmap View", XmSTRING_DEFAULT_CHARSET);
-		n = 0;
-		XtSetArg (args[n], XmNlabelString, str); n++;
-		mapvw_w = XmCreateToggleButton(fr_w, "BitmapView", args, n);
-		XtAddCallback(mapvw_w, XmNvalueChangedCallback, m_mapvw_cb, 0);
-		XtManageChild (mapvw_w);
-		XmStringFree (str);
-
-	    /* make the help button */
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 3); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 4); n++;
-	    w = XmCreatePushButton (f_w, "Help", args, n);
-	    XtAddCallback (w, XmNactivateCallback, m_help_cb, 0);
-	    XtManageChild (w);
-
-	/* make a separator over the control panel */
+	/* make the View pulldown */
 
 	n = 0;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, f_w); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	sep_w = XmCreateSeparator (mform_w, "MSep", args, n);
-	XtManageChild (sep_w);
+	pd_w = XmCreatePulldownMenu (mb_w, "ViewPD", args, n);
 
-	/* make the left rowcolumn, for the lib in lat/long */
+	    n = 0;
+	    XtSetArg (args[n], XmNsubMenuId, pd_w);  n++;
+	    XtSetArg (args[n], XmNmnemonic, 'V'); n++;
+	    cb_w = XmCreateCascadeButton (mb_w, "ViewCB", args, n);
+	    set_xmstring (cb_w, XmNlabelString, "View");
+	    XtManageChild (cb_w);
+
+	    /* make the earthshine toggle button */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    XtSetArg (args[n], XmNindicatorType, XmONE_OF_MANY); n++;
+	    eshine_w = XmCreateToggleButton (pd_w, "Earthshine", args, n);
+	    XtAddCallback (eshine_w, XmNvalueChangedCallback,m_eshine_cb,0);
+	    XtManageChild (eshine_w);
+
+	    /* make the view option toggle button */
+
+	    str = XmStringCreate("Bitmap View", XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    XtSetArg (args[n], XmNindicatorType, XmONE_OF_MANY); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    mapvw_w = XmCreateToggleButton(pd_w, "BitmapView", args, n);
+	    XtAddCallback(mapvw_w, XmNvalueChangedCallback, m_mapvw_cb, 0);
+	    XtManageChild (mapvw_w);
+	    XmStringFree (str);
+
+	    /* add a separator */
+
+	    n = 0;
+	    w = XmCreateSeparator (pd_w, "Sep", args, n);
+	    XtManageChild (w);
+
+	    /* add the More Info control */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "Stats", args, n);
+	    set_xmstring (w, XmNlabelString, "Libration info...");
+	    XtAddCallback (w, XmNactivateCallback, m_mstats_cb, NULL);
+	    XtManageChild (w);
+
+	/* make the help pulldown */
 
 	n = 0;
-	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
-	XtSetArg (args[n], XmNpacking, XmPACK_COLUMN); n++;
-	XtSetArg (args[n], XmNnumColumns, 2); n++;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, sep_w); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	rc_w = XmCreateRowColumn (mform_w, "MRCLeft", args, n);
-	XtManageChild (rc_w);
+	pd_w = XmCreatePulldownMenu (mb_w, "HelpPD", args, n);
 
-	    /* make the libration in lat/long columns */
-
-	    str = XmStringCreate ("Lib in Lat:", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
-	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    w = XmCreateLabel (rc_w, "MLLatL", args, n);
+	    XtSetArg (args[n], XmNsubMenuId, pd_w);  n++;
+	    XtSetArg (args[n], XmNmnemonic, 'H'); n++;
+	    cb_w = XmCreateCascadeButton (mb_w, "HelpCB", args, n);
+	    set_xmstring (cb_w, XmNlabelString, "Help");
+	    XtManageChild (cb_w);
+	    set_something (mb_w, XmNmenuHelpWidget, (XtArgVal)cb_w);
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "Help", args, n);
 	    XtManageChild (w);
-	    XmStringFree (str);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNuserData, "Moon.LibrLat"); n++;
-	    llat_w = XmCreatePushButton (rc_w, "MLLatPB", args, n);
-	    XtAddCallback (llat_w, XmNactivateCallback, m_activate_cb, 0);
-	    XtManageChild (llat_w);
-
-	    str = XmStringCreate ("Lib in Long:", XmSTRING_DEFAULT_CHARSET);
-	    n = 0;
-	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    w = XmCreateLabel (rc_w, "MLLongL", args, n);
-	    XtManageChild (w);
-	    XmStringFree (str);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNuserData, "Moon.LibrLong"); n++;
-	    llong_w = XmCreatePushButton (rc_w, "MLLongPB", args, n);
-	    XtAddCallback (llong_w, XmNactivateCallback, m_activate_cb, 0);
-	    XtManageChild (llong_w);
-
-	/* make the right rowcolumn, for the limb angle and total */
-
-	n = 0;
-	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
-	XtSetArg (args[n], XmNpacking, XmPACK_COLUMN); n++;
-	XtSetArg (args[n], XmNnumColumns, 2); n++;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, sep_w); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	rc_w = XmCreateRowColumn (mform_w, "MRC2", args, n);
-	XtManageChild (rc_w);
-
-	    /* make the limb/total columns */
-
-	    str = XmStringCreate ("Limb angle:", XmSTRING_DEFAULT_CHARSET);
-	    n = 0;
-	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    w = XmCreateLabel (rc_w, "MLimbL", args, n);
-	    XtManageChild (w);
-	    XmStringFree (str);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNuserData, "Moon.LibrLimb"); n++;
-	    limb_w = XmCreatePushButton (rc_w, "MLimbPB", args, n);
-	    XtAddCallback (limb_w, XmNactivateCallback, m_activate_cb, 0);
-	    XtManageChild (limb_w);
-
-	    str = XmStringCreate ("Tilt:", XmSTRING_DEFAULT_CHARSET);
-	    n = 0;
-	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    w = XmCreateLabel (rc_w, "MLibL", args, n);
-	    XtManageChild (w);
-	    XmStringFree (str);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNuserData, "Moon.LibrTilt"); n++;
-	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_END); n++;
-	    lib_w = XmCreatePushButton (rc_w, "MLibPB", args, n);
-	    XtAddCallback (lib_w, XmNactivateCallback, m_activate_cb, 0);
-	    XtManageChild (lib_w);
+	    XtAddCallback (w, XmNactivateCallback, m_help_cb, NULL);
 
 	/* make a label for the date stamp */
 
 	n = 0;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, rc_w); n++;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrecomputeSize, False); n++;
@@ -366,7 +310,8 @@ m_create_form()
 	/* make a frame for the drawing area */
 
 	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, mb_w); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNbottomWidget, dt_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
@@ -375,15 +320,161 @@ m_create_form()
 	frame_w = XmCreateFrame (mform_w, "MoonFrame", args, n);
 	XtManageChild (frame_w);
 
-	/* make a drawing area in the frame for the sky */
+	    /* make a drawing area in the frame for the sky */
+	    n = 0;
+	    XtSetArg (args[n], XmNwidth, MAPVW_W); n++;
+	    XtSetArg (args[n], XmNheight, MAPVW_H); n++;
+	    XtSetArg (args[n], XmNmarginWidth, 0); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    mda_w = XmCreateDrawingArea (frame_w, "MoonDA", args, n);
+	    XtManageChild (mda_w);
+	    XtAddCallback (mda_w, XmNexposeCallback, m_da_exp_cb, NULL);
+}
+
+static void
+m_create_msform()
+{
+	Widget rc_w;
+	Widget sep_w;
+	Widget w;
+	XmString str;
+	Arg args[20];
+	int n;
+
+	/* create form */
 	n = 0;
-	XtSetArg (args[n], XmNwidth, MAPVW_W); n++;
-	XtSetArg (args[n], XmNheight, MAPVW_H); n++;
-	XtSetArg (args[n], XmNmarginWidth, 0); n++;
-	XtSetArg (args[n], XmNmarginHeight, 0); n++;
-	mda_w = XmCreateDrawingArea (frame_w, "MoonDA", args, n);
-	XtManageChild (mda_w);
-	XtAddCallback (mda_w, XmNexposeCallback, m_da_exp_cb, 0);
+	XtSetArg (args[n], XmNautoUnmanage, False); n++;
+	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
+	msform_w = XmCreateFormDialog (toplevel_w, "MoonStats", args, n);
+
+	/* set some stuff in the parent DialogShell.
+	 * setting XmNdialogTitle in the Form didn't work..
+	 */
+	n = 0;
+	XtSetArg (args[n], XmNtitle, "xephem Moon info"); n++;
+	XtSetValues (XtParent(msform_w), args, n);
+
+	/* make a rowcolumn to hold the labels and info buttons */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNpacking, XmPACK_COLUMN); n++;
+	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
+	XtSetArg (args[n], XmNnumColumns, 4); n++;
+	XtSetArg (args[n], XmNisAligned, False); n++;
+	rc_w = XmCreateRowColumn (msform_w, "SRC", args, n);
+	XtManageChild (rc_w);
+
+	    /* make the libration in lat/long rows */
+
+	    str = XmStringCreate ("Libr in Lat:", XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    w = XmCreateLabel (rc_w, "MLLatL", args, n);
+	    XtManageChild (w);
+	    XmStringFree (str);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_END); n++;
+	    XtSetArg (args[n], XmNuserData, "Moon.LibrLat"); n++;
+	    llat_w = XmCreatePushButton (rc_w, "MLLatPB", args, n);
+	    XtAddCallback (llat_w, XmNactivateCallback, m_activate_cb, NULL);
+	    XtManageChild (llat_w);
+
+	    str = XmStringCreate ("Libr in Long:", XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    w = XmCreateLabel (rc_w, "MLLongL", args, n);
+	    XtManageChild (w);
+	    XmStringFree (str);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_END); n++;
+	    XtSetArg (args[n], XmNuserData, "Moon.LibrLong"); n++;
+	    llong_w = XmCreatePushButton (rc_w, "MLLongPB", args, n);
+	    XtAddCallback (llong_w, XmNactivateCallback, m_activate_cb, NULL);
+	    XtManageChild (llong_w);
+
+	    /* make the limb/tilt rows */
+
+	    str = XmStringCreate ("Limb angle:", XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    w = XmCreateLabel (rc_w, "MLimbL", args, n);
+	    XtManageChild (w);
+	    XmStringFree (str);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_END); n++;
+	    XtSetArg (args[n], XmNuserData, "Moon.LibrLimb"); n++;
+	    limb_w = XmCreatePushButton (rc_w, "MLimbPB", args, n);
+	    XtAddCallback (limb_w, XmNactivateCallback, m_activate_cb, NULL);
+	    XtManageChild (limb_w);
+
+	    str = XmStringCreate ("Tilt:", XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    w = XmCreateLabel (rc_w, "MLibL", args, n);
+	    XtManageChild (w);
+	    XmStringFree (str);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_END); n++;
+	    XtSetArg (args[n], XmNuserData, "Moon.LibrTilt"); n++;
+	    lib_w = XmCreatePushButton (rc_w, "MLibPB", args, n);
+	    XtAddCallback (lib_w, XmNactivateCallback, m_activate_cb, NULL);
+	    XtManageChild (lib_w);
+
+	/* add a separator */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, rc_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	sep_w = XmCreateSeparator (msform_w, "Sep2", args, n);
+	XtManageChild (sep_w);
+
+	/* add a label for the current date/time stamp */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, sep_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNalignment, XmALIGNMENT_CENTER); n++;
+	sdt_w = XmCreateLabel (msform_w, "SDTstamp", args, n);
+	XtManageChild (sdt_w);
+
+	/* add a separator */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, sdt_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	sep_w = XmCreateSeparator (msform_w, "Sep3", args, n);
+	XtManageChild (sep_w);
+
+	/* add a close button */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, sep_w); n++;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	XtSetArg (args[n], XmNleftPosition, 20); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+	XtSetArg (args[n], XmNrightPosition, 80); n++;
+	w = XmCreatePushButton (msform_w, "Close", args, n);
+	XtAddCallback (w, XmNactivateCallback, m_mstats_cb, NULL);
+	XtManageChild (w);
 }
 
 /* go through all the buttons pickable for plotting and set whether they
@@ -471,6 +562,24 @@ int whether;
 	XtSetValues (pbw, ap, na);
 }
 
+/* callback from the Close button is activated on the stats menu or when
+ * the More Info button is activated. they each do the same thing.
+ */
+/* ARGSUSED */
+static void
+m_mstats_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if ((mswasman = XtIsManaged(msform_w)))
+	    XtUnmanageChild (msform_w);
+	else {
+	    XtManageChild (msform_w);
+	    m_set_buttons(m_selecting);
+	}
+}
+
 /* callback from any of the data menu buttons being activated.
  */
 /* ARGSUSED */
@@ -482,7 +591,7 @@ XtPointer call;
 {
 	if (m_selecting) {
 	    char *name;
-	    get_something (w, XmNuserData, (char *)&name);
+	    get_something (w, XmNuserData, (XtArgVal)&name);
 	    register_selection (name);
 	}
 }
@@ -497,6 +606,8 @@ XtPointer client;
 XtPointer call;
 {
 	XtUnmanageChild (mform_w);
+	if ((mswasman = XtIsManaged(msform_w)))
+	    XtUnmanageChild (msform_w);
 }
 
 /* callback from the Map view toggle button.
@@ -598,12 +709,14 @@ int force;
 	    limb += 2*PI;	/* limb angle is traditionally 0..360 */
 	tilt = raddeg(sqrt(llat*llat + llong*llong));
 
-	f_dms_angle (llat_w, llat);
-	f_dms_angle (llong_w, llong);
-	f_dms_angle (limb_w, limb);
+
+	f_pangle (llat_w, llat);
+	f_pangle (llong_w, llong);
+	f_pangle (limb_w, limb);
 	f_double (lib_w, "%6.3f", tilt);
 
 	timestamp (np, dt_w);
+	timestamp (np, sdt_w);
 
 	if (XtIsManaged(mda_w))
 	    m_draw (force, limb);
@@ -673,14 +786,17 @@ double limb;
 	    Pixel p;
 
 	    gcm = GCForeground | GCBackground;
-	    get_something (mda_w, XmNforeground, (char *)&fg);
-	    get_something (mda_w, XmNbackground, (char *)&bg);
+	    get_something (mda_w, XmNforeground, (XtArgVal)&fg);
+	    get_something (mda_w, XmNbackground, (XtArgVal)&bg);
 	    gcv.background = fg;
 	    gcv.foreground = bg;
 	    m_fgc = XCreateGC (dsp, win, gcm, &gcv);
 
-	    /* make the libration marker gc */
-	    if (get_color_resource (mda_w, "MoonLibrColor", &p) < 0) {
+	    /* make the libration marker gc
+	     * beware of monochrome screens that allocate ok for any ol' color.
+	     */
+	    if (get_color_resource (mda_w, "MoonLibrColor", &p) < 0
+							|| p == bg || p == fg) {
 		gcm = GCForeground | GCFunction;
 		gcv.foreground = fg ^ bg;
 		gcv.function = GXxor;
@@ -694,7 +810,8 @@ double limb;
 
 	/* make an image copy so we can darken some of it */
 	m = (unsigned char *)XtMalloc(smallfm_width*smallfm_height/8);
-	(void) memcpy (m, smallfm_bits, smallfm_width*smallfm_height/8);
+	(void) memcpy ((void *)m, (void *)smallfm_bits,
+						smallfm_width*smallfm_height/8);
 
 	cosel = cos(el);
 	for (y = MRAD; y > -MRAD; y--) {
@@ -761,8 +878,8 @@ double limb;
 	    unsigned long fg, bg;
 	    Pixel p;
 
-	    get_something (mda_w, XmNforeground, (char *)&fg);
-	    get_something (mda_w, XmNbackground, (char *)&bg);
+	    get_something (mda_w, XmNforeground, (XtArgVal)&fg);
+	    get_something (mda_w, XmNbackground, (XtArgVal)&bg);
 
 	    gcm = GCForeground | GCBackground;
 	    gcv.foreground = fg;
@@ -773,8 +890,11 @@ double limb;
 	    gcv.background = fg;
 	    m_bgc = XCreateGC (dsp, win, gcm, &gcv);
 
-	    /* make the libration marker gc */
-	    if (get_color_resource (mda_w, "MoonLibrColor", &p) < 0) {
+	    /* make the libration marker gc
+	     * beware of monochrome screens that allocate ok for any ol' color.
+	     */
+	    if (get_color_resource (mda_w, "MoonLibrColor", &p) < 0
+							|| p == bg || p == fg) {
 		gcm = GCForeground | GCFunction;
 		gcv.foreground = fg ^ bg;
 		gcv.function = GXxor;
@@ -815,7 +935,7 @@ double limb;
 	XFillArc (dsp, pm, m_fgc, xb, yb, md, md, la1, la2);
 
 	/* draw the portion from the terminator to the meridian */
-	if (la1 < 0 && ta1 < 0 || la1 > 0 && ta1 > 0) {
+	if ((la1 < 0 && ta1 < 0) || (la1 > 0 && ta1 > 0)) {
 	    /* crescent, so draw in background color */
 	    XFillArc (dsp, pm, m_bgc, md/2-wid+xb, yb, 2*wid, md, ta1, ta2);
 	} else {
