@@ -17,12 +17,42 @@
 #include <Xm/RowColumn.h>
 #include <Xm/ScrollBar.h>
 #include <Xm/SelectioB.h>
+#include <Xm/Separator.h>
+#include <Xm/TextF.h>
+
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 #include "preferences.h"
 
 extern Widget toplevel_w;
 
+extern Now *mm_get_now P_((void));
+extern Obj *db_basic P_((int id));
+extern Obj *db_scan P_((DBScan *sp));
+extern char *obj_description P_((Obj *op));
+extern int db_n P_((void));
+extern int db_set_field P_((char bp[], int id, PrefDateFormat pref, Obj *op));
+extern int obj_cir P_((Now *np, Obj *op));
+extern void all_newobj P_((int dbidx));
+extern void db_scaninit P_((DBScan *sp));
+extern void db_setuserobj P_((int id, Obj *op));
+extern void f_double P_((Widget w, char *fmt, double f));
+extern void f_pangle P_((Widget w, double a));
+extern void f_ra P_((Widget w, double ra));
+extern void f_string P_((Widget w, char *s));
+extern void fs_date P_((char out[], double jd));
+extern void get_something P_((Widget w, char *resource, XtArgVal value));
+extern void get_xmstring P_((Widget w, char *resource, char **txtp));
+extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
+extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
+extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sv_id P_((Obj *op));
+extern void sv_point P_((Obj *op));
+extern void watch_cursor P_((int want));
+extern void zero_mem P_((void *loc, unsigned len));
+
 /* info about each field for a given type of object.
  * N.B. the prompt field is used to decide whether a button should be created
  *   to let this field be changed. Some prompts are determined at runtime
@@ -38,47 +68,6 @@ typedef struct {
     Widget pb_w;	/* PushButton if it can be changed, else Label */
 } Field;
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#define	Const const
-#else
-#define P_(s) ()
-#define	Const
-#endif
-
-extern Now *mm_get_now P_((void));
-extern Obj *db_basic P_((int id));
-extern void db_scaninit P_((DBScan *sp, HowScan how));
-extern Obj *db_scan P_((DBScan *sp));
-extern char *obj_description P_((Obj *op));
-extern int db_n P_((void));
-extern int db_set_field P_((char bp[], int id, PrefDateFormat pref, Obj *op));
-extern int obj_cir P_((Now *np, Obj *op));
-extern void sv_id P_((Obj *op));
-extern void all_newobj P_((int dbidx));
-extern void db_setuserobj P_((int id, Obj *op));
-extern void f_double P_((Widget w, char *fmt, double f));
-extern void f_pangle P_((Widget w, double a));
-extern void f_ra P_((Widget w, double ra));
-extern void f_string P_((Widget w, char *s));
-extern void fs_date P_((char out[], double jd));
-extern void get_something P_((Widget w, char *resource, char *value));
-extern void set_something P_((Widget w, char *resource, char *value));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void mjd_year P_((double Mjd, double *yr));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void sv_point P_((Obj *op));
-extern void watch_cursor P_((int want));
-extern void xe_msg P_((char *msg, int app_modal));
-extern void zero_mem P_((char *loc, unsigned len));
-
-void obj_manage P_((void));
-void obj_update P_((Now *np, int howmuch));
-void obj_set P_((Obj *op));
-void obj_newdb P_((int appended));
-void obj_cursor P_((Cursor c));
 static void init_working_obj P_((void));
 static void obj_unsetmenu P_((void));
 static void obj_set_from_db P_((Obj *op));
@@ -94,7 +83,9 @@ static void obj_xy_cb P_((Widget w, XtPointer client, XtPointer call));
 static void obj_type_cb P_((Widget w, XtPointer client, XtPointer call));
 static void obj_select_cb P_((Widget w, XtPointer client, XtPointer call));
 static void obj_scroll_cb P_((Widget w, XtPointer client, XtPointer call));
-static int nl_cmp P_((Const void *nl1, Const void *nl2));
+static void obj_srch_cb P_((Widget w, XtPointer client, XtPointer call));
+static int srch_match P_((char *n1, char *n2));
+static int nl_cmp P_((const void *nl1, const void *nl2));
 static void new_namelist P_((void));
 static void load_namelist P_((void));
 static void obj_setnames P_((int nli));
@@ -105,8 +96,6 @@ static void obj_set_button P_((Widget w, int id));
 static void epoch_as_decimal P_((Widget w, double e));
 static void epoch_as_mdy P_((Widget w, double e));
 
-#undef P_
-
 static Widget objform_w;	/* overall form Dialog */
 static Widget fldrc_w;		/* overall field info rc */
 static Widget scroll_w;		/* name scroll widget */
@@ -253,7 +242,7 @@ typedef struct {
 } NameList;
 
 #define	NBTNS	20		/* number of buttons in name list */
-static NameList *namelist;	/* malloced array of (db_n()-NOBJ) NameLists*/
+static NameList *namelist;	/* malloced array of db_n() NameLists*/
 static int nnamelist;		/* number of entries in namelist actually used*/
 static Widget namepb_w[NBTNS];	/* each name selection button */
 
@@ -284,14 +273,17 @@ obj_manage()
 	    XtManageChild (objform_w);
 }
 
-/* all this is really good for is if the date format changes.
- */
+/* update our view if we have been created */
 /* ARGSUSED */
 void
 obj_update (np, howmuch)
 Now *np;
 int howmuch;
 {
+	if (!objform_w)
+	    return;
+
+	new_namelist();
 	obj_setmenu();
 }
 
@@ -390,7 +382,7 @@ Obj *op;
 	}
 
 	/* *objp = *op; */
-	(void) memcpy ((char *)objp, (char *)op, sizeof(Obj));
+	(void) memcpy ((void *)objp, (void *)op, sizeof(Obj));
 }
 
 /* make the current working object officially in the database.
@@ -462,6 +454,7 @@ obj_create_form ()
 	Widget namerc_w, namercf_w;
 	Widget ctlfr_w, ctlf_w;
 	Widget skyfr_w, skyf_w;
+	Widget srchf_w, srchrc_w;
 	Widget w;
 	Arg args[20];
 	int n;
@@ -470,7 +463,6 @@ obj_create_form ()
 	/* create form */
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNallowOverlap, False); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
 	objform_w = XmCreateFormDialog (toplevel_w, "Obj", args, n);
@@ -510,41 +502,6 @@ obj_create_form ()
 		XtManageChild (w);
 	    }
 
-	/* make the overall field id rc in a frame.
-	 * the field id rc is just used as a manager to hold the several
-	 *   various type rcs; only one is ever on at once.
-	 */
-	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, typrbf_w); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	fldrcf_w = XmCreateFrame (objform_w, "FldFrame", args, n);
-	XtManageChild (fldrcf_w);
-
-	n = 0;
-	fldrc_w = XmCreateRowColumn (fldrcf_w, "FldInfoRC", args, n);
-	XtManageChild (fldrc_w);
-
-	    /* make one row/col for each possible Obj type.
-	     * they are managed later according to which type is displayed.
-	     * N.B. we assume the enum ObjType form indeces into typeinfo[]
-	     */
-	    for (i = UNDEFOBJ; i < PLANET; i++) {
-		TypeInfo *tp = &typeinfo[i];
-		Field *fp;
-
-		n = 0;
-		tp->frc_w = XmCreateRowColumn (fldrc_w, "TypeRC", args, n);
-
-		for (fp = tp->fp; fp < tp->fp + tp->nfields; fp++) {
-		    Widget fw;
-		    n = 0;
-		    fw = XmCreateForm (tp->frc_w, "ObjFldForm", args, n);
-		    obj_create_type_rc_form (fw, fp);
-		    XtManageChild (fw);
-		}
-	    }
-
 	/* make the x/y selection radio box in a frame */
 
 	n = 0;
@@ -572,7 +529,7 @@ obj_create_form ()
 							    (XtPointer)OBJY);
 	    XtManageChild (w);
 
-	/* make a rc in a frame to hold the name buttons */
+	/* make a rc in a frame to hold the name buttons and srch field */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
@@ -596,6 +553,39 @@ obj_create_form ()
 		namepb_w[i] = w;
 	    }
 
+	/* make the search label and text field in a frame */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, namercf_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNleftWidget, xyrbf_w); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	srchf_w = XmCreateFrame (objform_w, "SrchF", args, n);
+	XtManageChild (srchf_w);
+
+
+	    n = 0;
+	    srchrc_w = XmCreateRowColumn (srchf_w, "SrchRS", args, n);
+	    XtManageChild (srchrc_w);
+
+	    n = 0;
+	    w = XmCreateLabel (srchrc_w, "SrchL", args, n);
+	    set_xmstring (w, XmNlabelString, "Search:");
+	    XtManageChild (w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNmaxLength, MAXNM-1); n++;
+	    w = XmCreateTextField (srchrc_w, "SrchTF", args, n);
+	    XtManageChild (w);
+	    XtAddCallback (w, XmNactivateCallback, obj_srch_cb, NULL);
+
+#if XmVersion >= 1001
+	    /* init kb focus here if possible */
+	    XmProcessTraversal (w, XmTRAVERSE_CURRENT);
+	    XmProcessTraversal (w, XmTRAVERSE_CURRENT);
+#endif
+
 	/* make the scrollbar attached to the right edge */
 
 	n = 0;
@@ -614,7 +604,7 @@ obj_create_form ()
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, namercf_w); n++;
+	XtSetArg (args[n], XmNtopWidget, srchf_w); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
@@ -644,11 +634,50 @@ obj_create_form ()
 		XtManageChild (w);
 	    }
 
+	/* make the overall field id rc in a frame.
+	 * the field id rc is just used as a manager to hold the several
+	 *   various type rcs; only one is ever on at once.
+	 */
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, typrbf_w); n++;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
+	XtSetArg (args[n], XmNbottomWidget, namercf_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNrightWidget, namercf_w); n++;
+	fldrcf_w = XmCreateFrame (objform_w, "FldFrame", args, n);
+	XtManageChild (fldrcf_w);
+
+	n = 0;
+	fldrc_w = XmCreateRowColumn (fldrcf_w, "FldInfoRC", args, n);
+	XtManageChild (fldrc_w);
+
+	    /* make one row/col for each possible Obj type.
+	     * they are managed later according to which type is displayed.
+	     * N.B. we assume the enum ObjType form indeces into typeinfo[]
+	     */
+	    for (i = UNDEFOBJ; i < PLANET; i++) {
+		TypeInfo *tp = &typeinfo[i];
+		Field *fp;
+
+		n = 0;
+		tp->frc_w = XmCreateRowColumn (fldrc_w, "TypeRC", args, n);
+
+		for (fp = tp->fp; fp < tp->fp + tp->nfields; fp++) {
+		    Widget fw;
+		    n = 0;
+		    fw = XmCreateForm (tp->frc_w, "ObjFldForm", args, n);
+		    obj_create_type_rc_form (fw, fp);
+		    XtManageChild (fw);
+		}
+	    }
+
 	/* make a form in a frame for the sky point/id buttons */
 
 	n = 0;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, namercf_w); n++;
+	XtSetArg (args[n], XmNbottomWidget, srchf_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
 	XtSetArg (args[n], XmNrightWidget, xyrbf_w); n++;
@@ -876,7 +905,7 @@ XtPointer call;
 
 	if (t->set && objp->type != type) {
 	    obj_unsetmenu();
-	    zero_mem ((char *)objp, sizeof(Obj));
+	    zero_mem ((void *)objp, sizeof(Obj));
 	    objp->type = (Objtype_t) type;
 	    obj_setmenu();
 	}
@@ -896,7 +925,7 @@ XtPointer call;
 	int nameidx;
 	Obj *op;
 
-	get_something (w, XmNuserData, (char *)&nameidx);
+	get_something (w, XmNuserData, (XtArgVal)&nameidx);
 	if (nameidx < 0)
 	    return;	/* marked as unused */
 	if (nameidx >= nnamelist) {
@@ -925,12 +954,124 @@ XtPointer call;
 	obj_setnames (sp->value);
 }
 
+/* callback when CR is hit in the Srch test field.
+ */
+/* ARGSUSED */
+static void
+obj_srch_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	static char last_str[MAXNM];	/* last string we searched for */
+	static int last_idx = -1;	/* index to start */
+	char *str;
+	int i;
+	
+	str = XmTextFieldGetString (w);
+
+	/* start from top if new string or smaller list */
+	if (strcmp (str, last_str) || last_idx >= nnamelist) {
+	    last_idx = -1;
+	    (void) strcpy (last_str, str);
+	}
+	i = last_idx;
+
+	/* check list starting at i+1, wrapping back to last_idx */
+	for (;;) {
+
+	    i++;
+	    if (last_idx < 0 && i >= nnamelist) {
+		XBell (XtDisplay(w), 0);
+		break;
+	    }
+	    if (i >= nnamelist)
+		i = 0;
+
+	    if (srch_match (str, namelist[i].name)) {
+		int idx, max, slider;
+
+		/* find index to use for setting names and the scroll bar */
+		get_something (scroll_w, XmNmaximum, (XtArgVal)&max);
+		get_something (scroll_w, XmNsliderSize, (XtArgVal)&slider);
+		idx = i < max-slider ? i : max-slider;
+
+		/* set buttons so idxth entry appears on top */
+		obj_setnames (idx);
+
+		/* set slider to match */
+		set_something (scroll_w, XmNvalue, (XtArgVal)idx);
+
+		/* set up type menu with info about the selected item */
+		obj_unsetmenu();
+		obj_set_from_db(namelist[i].op);
+		obj_setmenu();
+
+		last_idx = i;
+		(void) strcpy (last_str, str);
+		break;
+	    }
+
+	    if (i == last_idx) {
+		last_idx = -1;
+		break;
+	    }
+	}
+
+	XtFree (str);
+}
+
+/* return 1 if n1 is anywhere in n2, ignoring case and whitespace;
+ * else return 0.
+ * N.B. we only use the first MAXNM-1 chars from each string and assume \0.
+ */
+static int
+srch_match (n1, n2)
+char *n1, *n2;
+{
+	char n1c[MAXNM];	/* n1 all upper case, no whitespace */
+	char n2c[MAXNM];	/* n2 all upper case, no whitespace */
+	int n1l, n2l;		/* lengths of n1c and n2c */
+	char c, *cp;
+	int i;
+
+	for (n1l = 0, cp = n1; (c = *cp++); ) {
+	    if (islower(c))
+		c = toupper (c);
+	    if (!isspace(c))
+		n1c[n1l++] = c;
+	}
+	n1c[n1l] = '\0';
+
+	for (n2l = 0, cp = n2; (c = *cp++); ) {
+	    if (islower(c))
+		c = toupper (c);
+	    if (!isspace(c))
+		n2c[n2l++] = c;
+	}
+	n2c[n2l] = '\0';
+
+	/* if n1 started with ^ it must match the start of n2 */
+	if (n1c[0] == '^')
+	    return (strncmp (n1c+1, n2c, n1l-1) == 0 ? 1 : 0);
+
+	for (i = 0; i <= n2l-n1l; i++)
+	    if (strncmp (n1c, n2c+i, n1l) == 0)
+		return (1);
+
+	return (0);
+}
+
 /* compare two NameList entries as required qsort(3).
  * take care to sort numerics by value.
+ * ignore case.
  */
 static int
 nl_cmp (nl1, nl2)
-Const void *nl1, *nl2;
+#if defined(__STDC__) || defined(__cplusplus)
+const
+#endif
+void *nl1, *nl2;
 {
 	char *s1 = ((NameList *)nl1)->name;
 	char *s2 = ((NameList *)nl2)->name;
@@ -939,7 +1080,11 @@ Const void *nl1, *nl2;
 
 	for (;;) {
 	    p = *s1++;
+	    if (islower(p))
+		p = toupper (p);
 	    q = *s2++;
+	    if (islower(q))
+		q = toupper (q);
 	    if (isdigit(p) && isdigit(q)) {
 		int np = 0, nq = 0;
 		do {
@@ -1002,14 +1147,16 @@ load_namelist ()
 	    nnamelist = 0;
 	}
 
-	nobj = db_n() - NOBJ;
+	nobj = db_n();
 	if (nobj <= 0)
 	    return;
 
 	nlp = namelist = (NameList *) XtMalloc (nobj*sizeof(NameList));
 
-	/* don't put any of the basic objects on the list */
-	for (db_scaninit(&dbs, SCAN_ALLBUT_PLANET); op = db_scan (&dbs); ) {
+	/* put everything except the basic objects on the list */
+	for (db_scaninit(&dbs); (op = db_scan (&dbs)); ) {
+	    if (op->type == PLANET || op->type == UNDEFOBJ)
+		continue;
 	    nlp->name = op->o_name;
 	    nlp->op = op;
 	    nlp++;
@@ -1017,7 +1164,7 @@ load_namelist ()
 
 	nnamelist = nlp - namelist;
 
-	qsort ((char *)namelist, nnamelist, sizeof(NameList), nl_cmp);
+	qsort ((void *)namelist, nnamelist, sizeof(NameList), nl_cmp);
 }
 
 /* fill the name buttons with the next NBTNS names starting at index nli 
@@ -1027,12 +1174,17 @@ static void
 obj_setnames(nli)
 int nli;
 {
+	static char noname[MAXNM];
 	XmString str;
 	Arg args[20];
 	char *namep;
 	int bi;		/* namepb_w[] index */
 	int nb;		/* number of good buttons to set */
 
+	/* noname is string of MAXNM-1 blanks for use when btn not used */
+	if (noname[0] == '\0')
+	    (void) sprintf (noname, "%*s", MAXNM-1, "");
+
 	nb = (nnamelist - nli < NBTNS) ? nnamelist - nli : NBTNS;
 	for (bi = 0; bi < NBTNS; bi++) {
 	    int n = 0;
@@ -1042,7 +1194,7 @@ int nli;
 		obj_set_a_button(namepb_w[bi], 1);
 		nli++;
 	    } else {
-		namep = " ";
+		namep = noname;
 		XtSetArg (args[n], XmNuserData, -1); n++;
 		obj_set_a_button(namepb_w[bi], 0);
 	    }
@@ -1142,7 +1294,7 @@ XtPointer call;
 	switch (s->reason) {
 	case XmCR_OK:
 	    get_xmstring(w, XmNtextString, &text);
-	    get_something (w, XmNuserData, (char *)&fp);
+	    get_something (w, XmNuserData, (XtArgVal)&fp);
 	    (void) db_set_field (text, fp->id,
 			    (PrefDateFormat) pref_get(PREF_DATE_FORMAT), objp);
 	    obj_set_button (fp->pb_w, fp->id);
@@ -1184,7 +1336,7 @@ Field *fp;
 	    XtUnmanageChild (w);
 	}
 
-	set_something (dw, XmNuserData, (char *)fp);
+	set_something (dw, XmNuserData, (XtArgVal)fp);
 	set_xmstring (dw, XmNselectionLabelString, fp->prompt);
 	set_xmstring(dw, XmNtextString, "");
 
