@@ -1,21 +1,9 @@
 #include <stdio.h>
 #include <math.h>
+
+#include "P_.h"
 #include "astro.h"
 
-
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
-
-extern void anomaly P_((double ma, double s, double *nu, double *ea));
-extern void pelement P_((double Mjd, double plan[8][9]));
-extern void range P_((double *v, double r));
-extern void sunpos P_((double Mjd, double *lsn, double *rsn));
-
-void plans P_((double mjd, int p, double *lpd0, double *psi0, double *rp0,
-    double *rho0, double *lam, double *bet, double *dia, double *mag));
 static void aux_jsun P_((double t, double *x1, double *x2, double *x3,
     double *x4, double *x5, double *x6));
 static void masun P_((double mjd, double *mas));
@@ -33,8 +21,6 @@ static void p_uranus P_((double t, double s, double *dl, double *dr,
 static void p_neptune P_((double t, double s, double *dl, double *dr,
     double *dml, double *ds, double *dm, double *da, double *dhl));
 
-#undef P_
-
 #define	TWOPI		(2*PI)
 #define	mod2PI(x)	((x) - (long)((x)/TWOPI)*TWOPI)
 
