@@ -15,39 +15,34 @@
 #include <Xm/Frame.h>
 #include <Xm/DrawingA.h>
 #include <Xm/Label.h>
+#include <Xm/CascadeB.h>
 #include <Xm/PushB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/Separator.h>
 #include <Xm/Text.h>
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
+#include "P_.h"
+
+extern Widget	toplevel_w;
+#define	XtD	XtDisplay(toplevel_w)
 
-extern int existsh P_((char *filename));
 extern FILE *fopenh P_((char *name, char *how));
 extern char *syserrstr P_((void));
+extern int existsh P_((char *filename));
 extern int plot_cartesian P_((FILE *pfp, Widget widget, unsigned int nx,
-    unsigned int ny, int flipx, int flipy, int grid));
+    unsigned int ny, int flipx, int flipy, int grid, int xasdate));
 extern void all_selection_mode P_((int whether));
 extern void f_string P_((Widget w, char *s));
-extern void get_something P_((Widget w, char *resource, char *value));
+extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
+extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void query P_((Widget tw, char *msg, char *label1, char *label2,
     char *label3, void (*func1)(), void (*func2)(), void (*func3)()));
-extern void set_something P_((Widget w, char *resource, char *value));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
 extern void xe_msg P_((char *msg, int app_modal));
 
-void plot_manage P_((void));
-void plt_selection P_((char *name));
-void plt_log P_((char *name, double value));
-void plot P_((void));
-int plot_ison P_((void));
-void plt_cursor P_((Cursor c));
 static void plt_select P_((int whether));
 static void plot_create_form P_((void));
 static void plt_activate_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -64,17 +59,13 @@ static void plt_try_turn_on P_((void));
 static void plt_turn_on P_((char *how));
 static void plt_da_manage P_((void));
 static void plt_da_destroy P_((Widget daform_w));
-static void plt_daform_map_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_flipx_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_flipy_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_grid_cb P_((Widget w, XtPointer client, XtPointer call));
+static void plt_da_xasdate_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_exp_cb P_((Widget da_w, XtPointer client, XtPointer call));
 
-#undef P_
-
-extern Widget toplevel_w;
-#define	XtD	XtDisplay(toplevel_w)
 
 /* max number of things we can keep track of at once to plot */
 #define	MAXPLTLINES	10
@@ -119,6 +110,7 @@ typedef struct {
     FILE *fp;		/* FILE pointer for the file */
     int flipx, flipy;	/* flip state for this instance */
     int grid;		/* whether to include a grid */
+    int xasdate;	/* whether to show x axis as dates */
 } DrawInfo;
 
 /* called when the plot menu is activated via the main menu pulldown.
@@ -209,6 +201,7 @@ plot()
 	}
 }
 
+int
 plot_ison()
 {
 	return (plt_fp != 0);
@@ -649,8 +642,7 @@ plt_da_manage()
 {
 	Widget daform_w;
 	Widget da_w, w;
-	Widget ctl_w;
-	Widget fr_w;
+	Widget mb_w, pd_w, cb_w;
 	XmString str;
 	Arg args[20];
 	char titlebuf[64];
@@ -672,44 +664,38 @@ plt_da_manage()
 	    return;
 	}
 
-	/* create the form dialog parent.
-	 * arrange for it to be square when it first comes up.
-	 */
+	/* create the form dialog parent. */
 	n = 0;
-	daform_w = XmCreateFormDialog (toplevel_w, "PlotD", args, n);
-	XtAddCallback (daform_w, XmNmapCallback, plt_daform_map_cb, NULL);
+	daform_w = XmCreateFormDialog (toplevel_w, "PlotDisplay", args, n);
+	XtAddCallback (daform_w, XmNmapCallback, prompt_map_cb, NULL);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
 	 */
-	(void) sprintf (titlebuf, "xephem Plot of `%.*s'", sizeof(titlebuf)-20,
-									    fn);
+	(void) sprintf (titlebuf, "xephem Plot of `%.*s'",
+						(int)sizeof(titlebuf)-20, fn);
 	n = 0;
 	XtSetArg (args[n], XmNtitle, titlebuf); n++;
 	XtSetValues (XtParent(daform_w), args, n);
 
 	/* make the DrawInfo structure and save it in the userData of the Form.
-	 * the memory gets freed when the the dialog is closed/unmanaged.
+	 * the memory gets freed when the dialog is closed/unmanaged.
+	 * the options get inited from their toggle buttons.
 	 */
 	di = (DrawInfo *) XtMalloc (sizeof(DrawInfo));
 	di->filename = fn;
 	di->fp = fp;
-	di->flipx = 0;
-	di->flipy = 0;
-	di->grid = 0;
-	set_something (daform_w, XmNuserData, (char *)di);
+	set_something (daform_w, XmNuserData, (XtArgVal)di);
 
-	/* make a form for the bottom controls */
+	/* create the menu bar across the top */
 
 	n = 0;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
-	XtSetArg (args[n], XmNfractionBase, 17); n++;
-	ctl_w = XmCreateForm (daform_w, "CtlF", args, n);
-	XtManageChild (ctl_w);
-
+	mb_w = XmCreateMenuBar (daform_w, "MB", args, n);
+	XtManageChild (mb_w);
+	
 	/* create the drawing area and connect plt_da_exp_cb().
 	 * N.B. be sure this guys parent is the FormDialog so exp_cb can find
 	 *   the DrawInfo by looking there at its userData.
@@ -718,9 +704,9 @@ plt_da_manage()
 	 *   assume that the daform_w is the parent of the DrawingArea.
 	 */
 	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, ctl_w); n++;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, mb_w); n++;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNbottomOffset, 2); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
@@ -730,75 +716,80 @@ plt_da_manage()
 	XtAddCallback (da_w, XmNexposeCallback, plt_da_exp_cb, NULL);
 	XtManageChild (da_w);
 
-	    /* make the flipx/y and grid toggle buttons (within frames) and the
-	     * close button in the control form
-	     */
+	/* make the Control pulldown */
+
+	n = 0;
+	pd_w = XmCreatePulldownMenu (mb_w, "ControlPD", args, n);
 
 	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 1); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 4); n++;
-	    fr_w = XmCreateFrame (ctl_w, "FlipXF", args, n);
-	    XtManageChild (fr_w);
+	    XtSetArg (args[n], XmNsubMenuId, pd_w);  n++;
+	    XtSetArg (args[n], XmNmnemonic, 'C'); n++;
+	    cb_w = XmCreateCascadeButton (mb_w, "ControlCB", args, n);
+	    set_xmstring (cb_w, XmNlabelString, "Control");
+	    XtManageChild (cb_w);
+
+	    /* make the close button.
+	     * it destroys the dialog and frees the DrawInfo struct.
+	     */
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "Close", args, n);
+	    XtAddCallback (w, XmNactivateCallback, plt_da_close_cb, daform_w);
+	    XtManageChild (w);
+
+	/* make the View pulldown */
+
+	n = 0;
+	pd_w = XmCreatePulldownMenu (mb_w, "ViewPD", args, n);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNsubMenuId, pd_w);  n++;
+	    XtSetArg (args[n], XmNmnemonic, 'V'); n++;
+	    cb_w = XmCreateCascadeButton (mb_w, "ViewCB", args, n);
+	    set_xmstring (cb_w, XmNlabelString, "View");
+	    XtManageChild (cb_w);
+
 	    str = XmStringCreate("Flip X", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
 	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    XtSetArg (args[n], XmNset, di->flipx); n++;
-	    w = XmCreateToggleButton(fr_w, "FlipX", args, n);
+	    w = XmCreateToggleButton(pd_w, "FlipX", args, n);
+	    di->flipx = XmToggleButtonGetState (w);
 	    XmStringFree (str);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_flipx_cb, da_w);
 	    XtManageChild (w);
 
-	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 5); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 8); n++;
-	    fr_w = XmCreateFrame (ctl_w, "FlipYF", args, n);
-	    XtManageChild (fr_w);
 	    str = XmStringCreate("Flip Y", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
 	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    XtSetArg (args[n], XmNset, di->flipy); n++;
-	    w = XmCreateToggleButton(fr_w, "FlipY", args, n);
+	    w = XmCreateToggleButton(pd_w, "FlipY", args, n);
+	    di->flipy = XmToggleButtonGetState (w);
 	    XmStringFree (str);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_flipy_cb, da_w);
 	    XtManageChild (w);
 
 	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 9); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 12); n++;
-	    fr_w = XmCreateFrame (ctl_w, "GridF", args, n);
-	    XtManageChild (fr_w);
-	    n = 0;
-	    XtSetArg (args[n], XmNset, di->grid); n++;
-	    w = XmCreateToggleButton(fr_w, "Grid", args, n);
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    w = XmCreateToggleButton(pd_w, "Grid", args, n);
+	    di->grid = XmToggleButtonGetState (w);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_grid_cb, da_w);
 	    XtManageChild (w);
 
-	    /* create a "Close" button.
-	     * it destroys the dialog and frees the DrawInfo struct.
-	     */
+	    str = XmStringCreate("Show X-Axis as Dates",
+						    XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 13); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 16); n++;
-	    w = XmCreatePushButton (ctl_w, "Close", args, n);
-	    XtAddCallback (w, XmNactivateCallback, plt_da_close_cb, daform_w);
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    w = XmCreateToggleButton(pd_w, "XAsDate", args, n);
+	    di->xasdate = XmToggleButtonGetState (w);
+	    XmStringFree (str);
+	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_xasdate_cb, da_w);
 	    XtManageChild (w);
-	
+
 	/* go. the expose will do the actual plotting */
 	XtManageChild (daform_w);
 }
@@ -813,7 +804,7 @@ Widget daform_w;
 {
 	DrawInfo *di;
 
-	get_something (daform_w, XmNuserData, (char *)&di);
+	get_something (daform_w, XmNuserData, (XtArgVal)&di);
 	(void) fclose (di->fp);
 	XtFree(di->filename);
 	XtFree((char *)di);
@@ -824,26 +815,6 @@ Widget daform_w;
 	XtDestroyWidget(daform_w);
 }
 
-/* called when a plot dialog is mapped.
- * set the height equal to the width.
- */
-/* ARGSUSED */
-static void
-plt_daform_map_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	Widget shell_w = XtParent(w);
-	Dimension width;
-	int iwidth;
-
-	get_something (shell_w, XmNwidth, (char *)&width);
-	iwidth = width;
-	set_something (shell_w, XmNheight, (char *)iwidth);
-}
-
-
 /* called when the Close button is pushed on a plot.
  * free the DrawInfo and destroy the dialog (passed as client).
  */
@@ -879,7 +850,7 @@ XtPointer call;
 	int x, y;
 	unsigned int nx, ny, bw, d;
 
-	get_something (daform_w, XmNuserData, (char *)&di);
+	get_something (daform_w, XmNuserData, (XtArgVal)&di);
 	di->flipx = t->set;
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
@@ -918,7 +889,7 @@ XtPointer call;
 	int x, y;
 	unsigned int nx, ny, bw, d;
 
-	get_something (daform_w, XmNuserData, (char *)&di);
+	get_something (daform_w, XmNuserData, (XtArgVal)&di);
 	di->flipy = t->set;
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
@@ -957,7 +928,7 @@ XtPointer call;
 	int x, y;
 	unsigned int nx, ny, bw, d;
 
-	get_something (daform_w, XmNuserData, (char *)&di);
+	get_something (daform_w, XmNuserData, (XtArgVal)&di);
 	di->grid = t->set;
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
@@ -974,6 +945,45 @@ XtPointer call;
 	XSendEvent (dsp, win, /*propagate*/False, ExposureMask, (XEvent *)&ev);
 }
 
+/* callback from the X-is-date toggle button within the drawing FormDiag itself.
+ * toggle the xisdate flag in the parent's DrawInfo structure and fake an
+ * expose. client is the DrawingArea widget.
+ */
+/* ARGSUSED */
+static void
+plt_da_xasdate_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	XmToggleButtonCallbackStruct *t = (XmToggleButtonCallbackStruct *) call;
+	Widget da_w = (Widget) client;
+	Widget daform_w = XtParent(da_w);
+	Display *dsp = XtDisplay(da_w);
+	Window win = XtWindow(da_w);
+	DrawInfo *di;
+	XExposeEvent ev;
+	Window root;
+	int x, y;
+	unsigned int nx, ny, bw, d;
+
+	get_something (daform_w, XmNuserData, (XtArgVal)&di);
+	di->xasdate = t->set;
+
+	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
+
+	ev.type = Expose;
+	ev.send_event = 1;	/* gets set anyways */
+	ev.display = dsp;
+	ev.window = win;
+	ev.x = ev.y = 0;
+	ev.width = nx;
+	ev.height = ny;
+	ev.count = 0;
+
+	XSendEvent (dsp, win, /*propagate*/False, ExposureMask, (XEvent *)&ev);
+}
+
 /* plot drawing area's expose callback.
  * redraw the graph to the (new?) size.
  * get a DrawInfo from our parent's userData.
@@ -1021,15 +1031,15 @@ XtPointer call;
 	 */
 
 	daform_w = XtParent(da_w);
-	get_something (daform_w, XmNuserData, (char *)&di);
+	get_something (daform_w, XmNuserData, (XtArgVal)&di);
 	XClearWindow (XtDisplay(da_w), XtWindow(da_w));
 	rewind (di->fp);
 	if (plot_cartesian (di->fp, da_w, xwa.width, xwa.height,
-					di->flipx, di->flipy, di->grid) < 0) {
+			    di->flipx, di->flipy, di->grid, di->xasdate) < 0) {
 	    /* had trouble, so done with this FormDialog.
 	     */
 	    char buf[128];
-	    (void) sprintf (buf, "Error plotting `%.*s'\n", sizeof(buf)-20,
+	    (void) sprintf (buf, "Error plotting `%.*s'\n", (int)sizeof(buf)-20,
 								di->filename);
 	    xe_msg (buf, 0);
 	    plt_da_destroy (daform_w);
