@@ -2,9 +2,11 @@
  * N.B. many of these enums are used as indexes -- don't change without
  *   checking where they are used!
  */
+#ifndef _PREFERENCES_H
+#define _PREFERENCES_H
 
 typedef enum {
-    PREF_ALGO, PREF_UNITS, PREF_DATE_FORMAT, PREF_TIMESTAMP_ZONE,
+    PREF_ALGO, PREF_UNITS, PREF_DATE_FORMAT, PREF_ZONE,
     PREF_DPYPREC, PREF_MSG_BELL
 } Preferences;
 
@@ -18,3 +20,5 @@ typedef enum {PREF_LOPREC, PREF_HIPREC} PrefDpyPrec;
 typedef enum {PREF_NOMSGBELL, PREF_MSGBELL} PrefMsgBell;
 
 int pref_get();
+
+#endif /* _PREFERENCES_H */
