@@ -19,9 +19,22 @@
 #include <Xm/Text.h>
 #include <Xm/Scale.h>
 #include <Xm/Separator.h>
+
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 
+extern Widget	toplevel_w;
+#define	XtD	XtDisplay(toplevel_w)
+
+extern Now *mm_get_now P_((void));
+extern void get_something P_((Widget w, char *resource, XtArgVal value));
+extern void gk_mag P_((double g, double k, double rp, double rho, double *mp));
+extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
+extern void sv_all P_((Now *np));
+extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
+    int y, int diam, int dotsonly));
+
 /* info for each filter option toggle */
 typedef struct {
     char *label;	/* label on the toggle button */
@@ -40,28 +53,10 @@ typedef struct {
     Widget pbw;		/* the category "toggle" pushbutton */
 } FilterCat;
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
-
-extern Now *mm_get_now P_((void));
-extern void get_something P_((Widget w, char *resource, char *value));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void sv_all P_((Now *np, int preclr));
-extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
-    int y, int diam, int dotsonly));
-
-void svf_create P_((void));
-void svf_manage_toggle P_((void));
-void svf_manage P_((void));
-void svf_unmanage P_((void));
-int svf_filter_ok P_((Obj *op));
-void svf_cursor P_((Cursor c));
 static void svf_create_filter P_((Widget mainrc, FilterCat *fcp));
 static void svf_reset P_((void));
 static void svf_apply P_((void));
+static int svf_bright_ok P_((Obj *op));
 static void svf_da_cb P_((Widget w, XtPointer client, XtPointer call));
 static void svf_draw_symbol P_((FilterTB *ftbp, Widget daw));
 static void svf_apply_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -72,10 +67,6 @@ static void svf_all_cb P_((Widget w, XtPointer client, XtPointer call));
 static void svf_reset_cb P_((Widget w, XtPointer client, XtPointer call));
 static void svf_cat_toggle_cb P_((Widget w, XtPointer client, XtPointer call));
 
-#undef P_
-
-extern Widget toplevel_w;
-#define	XtD	XtDisplay(toplevel_w)
 
 static FilterTB solsys_f[] = {
     {"Planets",		"Planets",	PLANET,		0},
@@ -87,6 +78,9 @@ static FilterTB solsys_f[] = {
 
 static FilterTB other_f[] = {
     {"Q Quasars", 	"Quasars",	FIXED,		'Q'},
+    {"L Pulsars", 	"Pulsars",	FIXED,		'L'},
+    {"J Radio", 	"Radio",	FIXED,		'J'},
+    {"R S/N Remnants", 	"SNRemnants",	FIXED,		'R'},
     {"T Stellar",	"Stellar",	FIXED,		'T'},
     {"Undefined",	"Undefined",	FIXED,		'\0'}
 };
@@ -133,11 +127,16 @@ static FilterCat filters[] = {
 static char fclass_table[128];
 static char type_table[10];
 
-static Widget filter_w;
+#define BMAGLIMIT       (-28)   /* brightest setting for the Mag scales */
+#define FMAGLIMIT       20      /* faintest setting for the Mag scales */
+static int fmag, bmag;		/* current values of faintest/brightest mag */
+static Widget fmag_w, bmag_w;	/* faintest and brightest mag scales */
+
+static Widget filter_w;		/* main filter dialog */
 
 /* create, but do not manage, the filter dialog.
  * we also apply its state info immediately to the initial resource
- * settings of the toggle buttons are used to set up the real filter
+ * settings of the toggle buttons since they are used to set up the real filter
  * right away.
  */
 void
@@ -157,6 +156,7 @@ svf_create()
 	Arg args[20];
 	XmString str;
 	Widget w;
+	Widget f_w;
 	Widget ctlf_w;
 	Widget topsep_w, botsep_w;
 	Widget hrc_w;
@@ -166,7 +166,7 @@ svf_create()
 	/* create form */
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
+	XtSetArg (args[n], XmNverticalSpacing, 4); n++;
 	filter_w = XmCreateFormDialog (toplevel_w, "SkyFilter", args, n);
 	XtAddCallback (filter_w, XmNmapCallback, prompt_map_cb, NULL);
 
@@ -177,12 +177,64 @@ svf_create()
 	XtSetArg (args[n], XmNtitle, "xephem Sky view Filter"); n++;
 	XtSetValues (XtParent(filter_w), args, n);
 
-	/* make a separator across the top */
+
+	/* put the mag limits in a form to center them across the top */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNfractionBase, 9); n++;
+	f_w = XmCreateForm (filter_w, "MagF", args, n);
+	XtManageChild (f_w);
+
+	    str = XmStringCreate ("Faintest Magnitude",
+						XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNleftPosition, 1); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNrightPosition, 4); n++;
+	    XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
+	    XtSetArg (args[n], XmNminimum, BMAGLIMIT); n++;
+	    XtSetArg (args[n], XmNmaximum, FMAGLIMIT); n++;
+	    XtSetArg (args[n], XmNprocessingDirection, XmMAX_ON_LEFT); n++;
+	    XtSetArg (args[n], XmNscaleMultiple, 1); n++;
+	    XtSetArg (args[n], XmNshowValue, True); n++;
+	    XtSetArg (args[n], XmNtitleString, str); n++;
+	    fmag_w = XmCreateScale (f_w, "FaintMagScale", args, n);
+	    XtManageChild (fmag_w);
+	    XmStringFree (str);
+
+	    str = XmStringCreate ("Brightest Magnitude",
+						XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNleftPosition, 5); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNrightPosition, 8); n++;
+	    XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
+	    XtSetArg (args[n], XmNminimum, BMAGLIMIT); n++;
+	    XtSetArg (args[n], XmNmaximum, FMAGLIMIT); n++;
+	    XtSetArg (args[n], XmNprocessingDirection, XmMAX_ON_LEFT); n++;
+	    XtSetArg (args[n], XmNscaleMultiple, 1); n++;
+	    XtSetArg (args[n], XmNshowValue, True); n++;
+	    XtSetArg (args[n], XmNtitleString, str); n++;
+	    bmag_w = XmCreateScale (f_w, "BrightMagScale", args, n);
+	    XtManageChild (bmag_w);
+	    XmStringFree (str);
+
+	/* make a separator below the mag scales */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, f_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	topsep_w = XmCreateSeparator (filter_w, "TopSep", args, n);
 	XtManageChild (topsep_w);
 
@@ -250,20 +302,16 @@ svf_create()
 	svf_apply();
 }
 
-/* called to toggle whether the filter dialog is up.
- * whenever we are managed, reset the state of the toggle buttons back
- * to the way the real filter is.
+/* return 1 if we are now up, else 0.
  */
-void
-svf_manage_toggle ()
+int
+svf_ismanaged()
 {
-	if (XtIsManaged(filter_w))
-	    svf_unmanage();
-	else
-	    svf_manage();
+
+	return (XtIsManaged(filter_w));
 }
 
-/* called to manage the filer dialog.
+/* called to manage the filter dialog.
  * whenever we are managed, reset the state of the toggle buttons back
  * to the way the real filter is.
  */
@@ -274,25 +322,42 @@ svf_manage()
 	XtManageChild(filter_w);
 }
 
+/* called to manage the filter dialog.
+ */
 void
 svf_unmanage()
 {
 	XtUnmanageChild(filter_w);
 }
 
+/* return the current faint and bright mag limits */
+void
+svf_maglimits (fmagp, bmagp)
+int *fmagp, *bmagp;
+{
+	*fmagp = fmag;
+	*bmagp = bmag;
+}
+
 /* return 1 if p satisfies the stuff in the filter dialog, else 0.
  * N.B. because svf_apply() has to have already been called we do not need
  *   to range check the type and class.
  */
+int
 svf_filter_ok (op)
 Obj *op;
 {
 	int t = op->type;
+	int tok;
 
 	if (t == FIXED)
-	    return (fclass_table[op->f_class]);
+	    tok = fclass_table[(int)op->f_class];
 	else
-	    return (type_table[t]);
+	    tok = type_table[t];
+
+	if (tok)
+	    return (svf_bright_ok (op));
+	return (0);
 }
 
 /* called to put up or remove the watch cursor.  */
@@ -398,6 +463,7 @@ FilterCat *fcp;
 }
 
 /* set the toggle button states according to the real filter states.
+ * set the mag scales from the real mag limits.
  */
 static void
 svf_reset()
@@ -411,9 +477,13 @@ svf_reset()
 		XmToggleButtonSetState (ftbp->tbw, ftbp->state, True);
 	    }
 	}
+
+	XmScaleSetValue (fmag_w, fmag);
+	XmScaleSetValue (bmag_w, bmag);
 }
 
 /* set the filter according to the present state of the toggle buttons.
+ * also set the mag limits from the scales.
  */
 static void
 svf_apply()
@@ -441,6 +511,53 @@ svf_apply()
 		    type_table[t] = ftbp->state;
 	    }
 	}
+
+	XmScaleGetValue (fmag_w, &fmag);
+	XmScaleGetValue (bmag_w, &bmag);
+}
+
+/* return 1 if db object is ever possibly within the fmag/bmag range; else 0.
+ */
+static int
+svf_bright_ok(op)
+Obj *op;
+{
+	switch (op->type) {
+	case PLANET:
+	    /* always go for the planets for now but ..
+	     * TODO: work up a table of extreme planet magnitudes.
+	     */
+	    return (1);
+	    /* break; */
+	case HYPERBOLIC: return (1);	/* and interlopers */
+	case PARABOLIC: return (1);
+	case ELLIPTICAL: {
+	    double mag;		/* magnitude */
+	    double per, aph;	/* perihelion and aphelion distance */
+
+	    per = op->e_a*(1.0 - op->e_e);
+	    aph = op->e_a*(1.0 + op->e_e);
+	    if (per <= 1.1 && aph >= 0.9)
+		return (1); /* might be blazing in the back yard some day */
+	    if (op->e_mag.whichm == MAG_HG)
+		 mag = op->e_mag.m1 + 5*log10(per*fabs(per-1.0));
+	    else
+		 gk_mag(op->e_mag.m1, op->e_mag.m2,
+						per, fabs(per-1.0), &mag);
+	    return (mag <= fmag && mag >= bmag);
+	    /* break; */
+	    }
+	case FIXED:
+	    return (op->f_mag/MAGSCALE <= fmag && op->f_mag/MAGSCALE >= bmag);
+	    /* break; */
+	case EARTHSAT:
+	    /* TODO: work on satellite magnitudes someday */
+	    return (1);
+	default: 
+	    printf ("sv_bright_ok(): bad type: %d\n", op->type);
+	    exit (1);
+	    return (0);	/* for lint */
+	}
 }
 
 /* called when a symbol drawing area is exposed.
@@ -502,9 +619,9 @@ Widget daw;
 	if (ftbp->type == FIXED)
 	    obj.f_class = ftbp->class;
 
-	get_something (daw, XmNwidth, (char *)&w);
-	get_something (daw, XmNheight, (char *)&h);
-	diam = (w > h ? h : w) - 2;
+	get_something (daw, XmNwidth, (XtArgVal)&w);
+	get_something (daw, XmNheight, (XtArgVal)&h);
+	diam = (w > h ? h : w) - 2; /* allow for a small border */
 
 	if (!gc) {
 	    XGCValues gcv;
@@ -512,13 +629,12 @@ Widget daw;
 	    Pixel p;
 
 	    gcm = GCForeground;
-	    get_something (daw, XmNforeground, (char *)&p);
+	    get_something (daw, XmNforeground, (XtArgVal)&p);
 	    gcv.foreground = p;
 	    gc = XCreateGC (dsp, win, gcm, &gcv);
 	}
 
-	/* use diam-2 to allow for a little overflow on some symbols */
-	sv_draw_obj (dsp, win, gc, &obj, (int)w/2, (int)h/2, diam-2, 0);
+	sv_draw_obj (dsp, win, gc, &obj, (int)w/2-1, (int)h/2-1, diam, 0);
 	sv_draw_obj (dsp, win, gc, NULL, 0, 0, 0, 0);	/* flush */
 }
 
@@ -531,7 +647,7 @@ XtPointer client;
 XtPointer call;
 {
 	svf_apply();
-	sv_all(mm_get_now(), 1);
+	sv_all(mm_get_now());
 }
 
 /* called when Ok is pushed on the filter dialog */
@@ -543,7 +659,7 @@ XtPointer client;
 XtPointer call;
 {
 	svf_apply();
-	sv_all(mm_get_now(), 1);
+	sv_all(mm_get_now());
 	XtUnmanageChild (filter_w);
 }
 
