@@ -20,14 +20,13 @@
 #include <Xm/SelectioB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/Text.h>
+
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
+extern Widget	toplevel_w;
+#define	XtD	XtDisplay(toplevel_w)
 
 extern Now *mm_get_now P_((void));
 extern int any_ison P_((void));
@@ -36,9 +35,8 @@ extern int execute_expr P_((double *vp, char *errbuf));
 extern int prog_isgood P_((void));
 extern void all_selection_mode P_((int whether));
 extern void compiler_log P_((char *name, double value));
-extern void f_dec_sexsign P_((double x, int *h, int *m, int *s));
 extern void f_double P_((Widget w, char *fmt, double f));
-extern void f_sscansex P_((char *bp, int *d, int *m, int *s));
+extern void f_scansex P_((double o, char bp[], double *np));
 extern void f_time P_((Widget w, double t));
 extern void get_xmstring P_((Widget w, char *resource, char **txtp));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
@@ -46,18 +44,10 @@ extern void lst_selection P_((char *name));
 extern void plt_selection P_((char *name));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void redraw_screen P_((int how_much));
-extern void set_something P_((Widget w, char *resource, char *value));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sex_dec P_((int hd, int m, int s, double *d));
 extern void xe_msg P_((char *msg, int app_modal));
 
-void srch_manage P_((void));
-void srch_selection_mode P_((int whether));
-void srch_selection P_((char *name));
-int srch_eval P_((double Mjd, double *tmincp));
-int srch_ison P_((void));
-void srch_log P_((char *name, double value));
-void srch_cursor P_((Cursor c));
 static void srch_create_form P_((void));
 static srch_isup P_((void));
 static srching_now P_((void));
@@ -77,10 +67,6 @@ static srch_minmax P_((double Mjd, double v, double *tmincp));
 static srch_solve0 P_((double Mjd, double v, double *tmincp));
 static srch_binary P_((double Mjd, double v, double *tmincp));
 
-#undef P_
-
-extern Widget toplevel_w;
-#define	XtD	XtDisplay(toplevel_w)
 
 /* the widgets we need direct access to */
 static Widget srchform_w;
@@ -140,8 +126,8 @@ int whether;	/* whether setting up for plotting or for not plotting */
 	srch_selecting += whether ? 1 : -1;
 
 	if (srchform_w && XtIsManaged(srchform_w))
-	    if (whether && srch_selecting == 1     /* first one to want on */
-		|| !whether && srch_selecting == 0 /* last one to want off */)
+	    if ((whether && srch_selecting == 1)     /* first one to want on */
+		|| (!whether && srch_selecting == 0) /* last one to want off */)
 		srch_set_buttons (whether);
 }
 
@@ -184,6 +170,7 @@ char *name;
  *   search function (if it is ok) and flog and display it.
  * return 0 if caller can continue or -1 if it is time to stop.
  */
+int
 srch_eval(Mjd, tmincp)
 double Mjd;
 double *tmincp;
@@ -220,6 +207,7 @@ double *tmincp;
  * we say True whenever there is a validly compiled function and either we are
  *   searching (obviously) or the srch control menu is up.
  */
+int
 srch_ison()
 {
 	return (prog_isgood() && (srching_now() || srch_isup()));
@@ -353,7 +341,7 @@ srch_create_form()
 	compile_w = XmCreatePushButton (rc_w, "Compile", args, n);
 	XtAddCallback (compile_w, XmNactivateCallback, srch_compile_cb, 0);
 	XtManageChild (compile_w);
-	set_something (srchform_w, XmNdefaultButton, (char *)w);
+	set_something (srchform_w, XmNdefaultButton, (XtArgVal)w);
 
 	/* create a separator */
 
@@ -457,7 +445,7 @@ srch_create_form()
 	    /* add the name in the userData field in case it's plotted or
 	     * listed
 	     */
-	    set_something (valu_w, XmNuserData, srchvname);
+	    set_something (valu_w, XmNuserData, (XtArgVal)srchvname);
 
 	/* button to select using for plotting. 
 	 * ONLY MANAGED WHEN WE ARE TOLD TO SELECT FOR PLOTTING.
@@ -505,7 +493,7 @@ srch_create_form()
 }
 
 /* return True whenever the srch control menu is up */
-static
+static int
 srch_isup()
 {
 	return (srchform_w && XtIsManaged(srchform_w));
@@ -513,7 +501,7 @@ srch_isup()
 
 /* return True whenever we are actually in the midst of controlling a search.
  */
-static
+static int
 srching_now()
 {
 	return (on_w && XmToggleButtonGetState(on_w));
@@ -672,14 +660,11 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	int hrs, mins, secs;
 	char *text;
 	
-	f_dec_sexsign (tmlimit, &hrs, &mins, &secs);
 	get_xmstring(w, XmNtextString, &text);
-	f_sscansex (text, &hrs, &mins, &secs);
+	f_scansex (tmlimit, text, &tmlimit);
 	XtFree (text);
-	sex_dec (hrs, mins, secs, &tmlimit);
 	f_time (acc_w, tmlimit);
 	XtUnmanageChild (w);
 }
@@ -710,7 +695,7 @@ XtPointer call;
 	    XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	    XtSetArg(args[n], XmNselectionLabelString, str);  n++;
 	    XtSetArg(args[n], XmNdialogTitle, title);  n++;
-	    dw = XmCreatePromptDialog(toplevel_w, "xephem Accuracy", args, n);
+	    dw = XmCreatePromptDialog(toplevel_w, "Accuracy", args, n);
 	    XtAddCallback (dw, XmNokCallback, srch_acc_ok_cb, NULL);
 	    XtAddCallback (dw, XmNmapCallback, prompt_map_cb, NULL);
 	    XmStringFree (str);
@@ -789,7 +774,7 @@ XtPointer call;
 /* use successive paraboloidal fits to find when expression is at a
  * local minimum or maximum.
  */
-static
+static int
 srch_minmax(Mjd, v, tmincp)
 double Mjd;
 double v;
@@ -873,7 +858,7 @@ double *tmincp;
 
 /* use secant method to solve for time when expression passes through 0.
  */
-static
+static int
 srch_solve0(Mjd, v, tmincp)
 double Mjd;
 double v;
@@ -908,7 +893,7 @@ double *tmincp;
  * if the change is outside the initial tminc range, then keep searching in that
  *    direction by tminc first before starting to divide down.
  */
-static
+static int
 srch_binary(Mjd, v, tmincp)
 double Mjd;
 double v;
