@@ -2,10 +2,169 @@
  * first thing is our modification history, then code to put up the dialog.
  */
 
-/* add a note after each change.
- */
-
 /*
+ * 2.7.1 8/10	solsys: fix stereo expose and extra ecliptic lines.
+ *		clean up help for Triad format and some typos.
+ *		skyview: fix overlapping RA/Az grid marks near poles.
+ *		formats: fix times near 24:00
+ *	8/11	earthmenu: improve changing from cyl <-> sph
+ *	8/17	dbmenu: slight change for Ultrix.
+ *		solsys: improve help for stereo.
+ *	8/19	trails: tweak the character set just a bit.
+ *	8/21	jupmenu: delete unused variable.
+ * 2.7	5/18	add close objects facility.
+ *	5/19	db.c: no "/" between filename components #ifdef VMS
+ *	5/22	sky filt: slightly larger symbols and tweak up centering.
+ *	5/23	sky view: more tweaking of grid spacing.
+ *	5/24	satmenu: display database objects behind saturn.
+ *	5/26	allow fractional seconds in all ra/dec entries.
+ *	5/30	clean up external function declarations (touched all files).
+ *		scan TZName/TZone first so other fields may use when no OS sprt.
+ *		data menu: add earth- and sun-object light travel time columns.
+ *	5/31	saturn view: add more features.
+ *		help menu: don't rely on XmText widget to expand tabs.
+ *	6/1	sky view: streamline trail setup with its own dialog.
+ *	6/2	sky view: add trail help.
+ *	6/13	support new fixed class L for Pulsars and add a sample database.
+ *	6/14	minor changes for sunos.
+ *		add otherStellarColor resource.
+ *		tweak pulsar symbol.
+ *	6/15	skyviewmenu: add "Point + Zoom" to pulldown.
+ *		reading from DBFIFO no longer pre-deletes back to checkpoint.
+ *	6/19	closemenu: add Auto option.
+ *		add a simple progress meter.
+ *	6/20	dbmenu: open DBFIFO each time the button is pressed.
+ *	6/21	dbmenu: add No Dups feature.
+ *	6/22	support new fixed class J for Radio and add sample vla databases
+ *	6/26	dbmenu: include user-defined objects in tallys now too.
+ *	6/28	earthsat: PI2 conflicted with one in linux's math.h.
+ *		jupmenu: add features much like satmenu.
+ *	6/29	skyview: add zoom factors to popup.
+ *	6/30	skyview: move Locate to main menu bar.
+ *		jupmenu: add GRS
+ *		closing also implies turning off movie demos.
+ *	7/4	skyview: add file list control.
+ *	7/7	closing views also stops movies.
+ *	7/8	data: allow user input of rise/set horizon displacement.
+ *	7/10	close: tweak up algorithm, add file write option.
+ *	7/11	main: LT "Mightnight Tonight" shortcut and zone name under cal.
+ *		add What's up help.
+ *	7/12	sky view: added Go back view command
+ *		earthmenu: fine tune drawing precision.
+ *		earthmenu: trails now connected and use tickmarks like sky view.
+ *		closemenu: discard objects below horizon when in topo mode.
+ *	7/14	main: fix display of negative timezone and step sizes.
+ *	7/15	add trail control (ala sky view) to earth too.
+ *	7/16	earthmenu: fix Create Trail prompt.
+ *		xephem.c: add basic Trail fallbacks.
+ *		solsys: add watch cursor (!)
+ *	7/17	misc changes for UnixWare C compiler.
+ *		retain trails setup between calls.
+ *	7/20	trails: add control over rounding.
+ *	7/22	start work on rotated text for trails.
+ *	8/3	add more trail options.
+ *	8/4	earthmenu: fine-tune circle drawing a bit more.
+ *		solsys: graft new trail facility here too.
+ *	8/6	solsys: fix trail effect on current object circumstances.
+ *		skyview: add explicit Save to compliment Go back.
+ *		earth: don't discard trails at Update if Show Trail is on.
+ *	8/7	skyview: add richer history facility than just goback.
+ * 2.6.3 4/26	set/get_something(): change value from (char *) to (XtArgVal)
+ *	5/1	allow ! to introduce a message coming over DBFIFO.
+ *		add gscdbd to Manifest.
+ *	5/9	add (void *) casts to mem* function args.
+ *		XEphem.ad: *background to gray30 and other minor changes.
+ *		earth view: add crude+fast option
+ *		sky view: allow .1 degree FOV and pointing control.
+ *		sky view: allow constellation names without figures.
+ *	5/10	sky view: tweak up grid for very small fov.
+ *	5/11	sky view: add labels to grid.
+ *	5/17	saturn info: add ring tilt from sun (and improve algorithm).
+ *		plot: option to draw X axis as dates (not just decimal years)
+ *	5/18	fs_date(): allow for %g rounding up day
+ *		sky view: send to LOCFIFO via Button3 popup, not Button2.
+ *		sky view: use mediumStarColor when spectral class unknown.
+ *		start breaking up some help sections into smaller chunks.
+ * 2.6.2 1/30	earthmenu.c, line 448: init sites_w (fixes bombing if no sites).
+ *	2/28	add comments explaining TZone and TZName to XEphem.ad.
+ *		remove obsolete entries for AltAzMap in XEphem.ad.
+ *		give a better explanation of discarding sol sys trails in help.
+ *		sky and earth views close properly from direct windw mngr cmd.
+ *		improve mktime() in time.c (thanks to madler@vssad.enet.dec.com)
+ *		surround all colors in XEphem.ad with #ifdef COLOR for xrdb.
+ *		add help comment for dawn/dusk needing reasonable TZ Offset.
+ * 	4/24	libration.c: change some statics to #defines.
+ *		skyviewmenu.c: make popup child of svda_w (fixes X Grab error)
+ *		solsysmenu.c: make popup child of svss_w (fixes X Grab error)
+ * 2.6.1 1/23	minor portability patches for VMS, OSF/1
+ *		fix bug when XEphem.DBinitialFiles was set to nothing.
+ *	1/24	scrub xephem.hlp for typos
+ *		fixed skyview initial garbage and earthview new-mode half-paint.
+ * 2.6  4/11	add Movie shortcuts to some of the graphical views for demos.
+ *		message menu now starts in an unmapped state.
+ *	4/13	various performance improvments: SkyView is 25% faster now.
+ *      5/3	display all timestamp updates, even during movie loops.
+ *	5/12	draw Sun and Moon to actual size in SkyView (great for eclipses)
+ *      8/2	improve earth trig near poles.
+ *		improve anomaly.c (thanks to Richard Clark).
+ *      8/4	allow searching for db object by name.
+ *		improve stability of ObjX/Y menu sizing behavior.
+ *		add top/bottom and left/right flip controls to sky view.
+ *	8/7	improve placement of plot labels.
+ *	8/23	continue object name searches from where left off last.
+ *		add display of LMT (UT - radhr(LNG)) to earth popup and tracking
+ *	8/31	add cylindrical projection to earth menu.
+ *	9/27	updated ephem.db with epoch 10/5/1994 asteroids.
+ *	10/2	fix coord tracking bug in earthview.
+ *	11/2	improved earth map data.
+ *		improve stability of earthview menu sizing behavior.
+ *		add N/S and E/W flippers to saturn and jupiter views.
+ *		just draw bullseye for newest trailing entry in earthview.
+ *	11/14	show solar eclipse path projection on earth view.
+ *		s_edist is now in AU throughout (and added constants to support)
+ *		add display of LST (GST- radhr(LNG)) to earth popup and tracking
+ *		gray out fields that are not updated when looping with Pause=0
+ *		Timezone preference now effects all time fields, not just stamp.
+ *		Auto timezone tracking mode (maintains local conventions)
+ *		support for longer time zone names (up to 7 characters)
+ *	11/16	add constellation figures to skyview.
+ *	11/17	Pause is now a float number of seconds.
+ *	11/21	support fixed class "R": supernova remnant
+ *		improve some of the skyviewmenu object symbols.
+ *		add size to sky view popup menu.
+ *		ignore case when sorting objx/y list.
+ *		add a basic set of built-in objects.
+ *		provide a general-purpose file selection box to manage db files.
+ *		break up ephem.db into several *.edb files.
+ *	11/25	add angular separation to skyview tracking.
+ *		add FOV and limiting mag to loc_fifo message.
+ *		improved support for finite sized objects.
+ *		retain az better at zenith if switch to ra/dec and right back.
+ *	12/5	faster object culling in skyviewmenu.
+ *	12/8	don't init aggregate in earthmenu.c (choked a Sun compiler).
+ *		fix constellation figures coords that had dec < 0.
+ *		round skyview symbol sizes down from just dots, not up.
+ *		remove use of HUGE from misc.c (outdated and some are bogus).
+ *	12/12	report fractional days in some dates (fs_date(); +mjd_day()).
+ *		fix spacecraft.edb.
+ *		set skyview FOV scaleMultiple=10 and try to give it KB focus.
+ *	12/13	ignore xephem_sites lines that do not begin with alpha or digit.
+ *		change to DBdirectory/DBpattern/DBinitialFiles; drop dbbuiltin.c
+ *		fix bug in faster skyviewmenu object culling.
+ *		retain ra better at poles if switch to alt/az and right back.
+ *	12/16	change layout of most views to use menu bar.
+ *	12/28	lots more menu layout changes.
+ *	12/29	fix bug in hadec_aa() when lat was near either pole.
+ *	12/30	make skyview trailed objects immune to filtering.
+ *      1/5/95	menu bar for db menu too.
+ *		move opening of DB fifo into db menu with it's own button
+ *	1/6	change round earth projection name from polar to spherical.
+ *	1/12	switched to Dan Bruton's code for saturn's moons and ring tilt.
+ *	1/13	eliminate not-so-portable union initialization from earthmenu.c
+ *		use menu bar in plot view too.
+ *	1/16	improved constants for Iapetus elements.
+ *		fix bug in hours up when using UTZ timezone preference.
+ * 2.5a 4/6/94	mods to plot_aux.c and moonmenu.c for monochrome displays.
  * 2.5  6/8	make mainmenu prompt shortcuts context-sensitive.
  *		get timezone name and offset during "Now" ifdef __STDC__.
  *		increase max timezone name length from 3 to 4.
@@ -110,21 +269,18 @@
 #include <Xm/ToggleB.h>
 #include <Xm/Text.h>
 #include <Xm/Scale.h>
+
+#include "P_.h"
 #include "astro.h"
 #include "circum.h"
 #include "patchlevel.h"
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
+extern Widget toplevel_w;
+extern XtAppContext xe_app;
 
-extern void set_something P_((Widget w, char *resource, char *value));
-extern void get_something P_((Widget w, char *resource, char *value));
+extern void get_something P_((Widget w, char *resource, XtArgVal value));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
 
-void version P_((void));
-void v_cursor P_((Cursor c));
 static void v_makew P_((void));
 static void fill_msg P_((Widget w));
 static void v_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -137,16 +293,11 @@ static void drawPlanet P_((Display *dsp, Window win, GC gc, int sx, int sy,
     int w, int h));
 static void v_define_fgc P_((void));
 
-#undef P_
-
-extern Widget toplevel_w;
-extern XtAppContext xe_app;
-
 static char *msg[] = {
 "xephem - An Interactive Astronomical Ephemeris Program for X",
 PATCHLEVEL,
 "",
-"Copyright (c) 1990,1991,1992,1993 by Elwood Charles Downey",
+"Copyright (c) 1990,1991,1992,1993,1994,1995 by Elwood Charles Downey",
 "ecdowney@noao.edu",
 "",
 "Permission is granted to make and distribute copies of this program free of",
@@ -214,7 +365,7 @@ static int delta_area;		/* change in area per step, sqr pixels */
 static int perihelion;		/* min dist from sun, pixels */
 static int maxtail;		/* max tail len (ie, tail@peri), pixels */
 
-/* called when mainmenu "on Version" help is selected.
+/* called when mainmenu "About.." help is selected.
  */
 void
 version()
@@ -274,14 +425,14 @@ v_makew()
 	XtSetArg (args[n], XmNautoUnmanage, True); n++;
 	XtSetArg (args[n], XmNhorizontalSpacing, 5); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
-	v_w = XmCreateFormDialog (toplevel_w, "Version", args, n);
+	v_w = XmCreateFormDialog (toplevel_w, "About", args, n);
 	XtAddCallback (v_w, XmNunmapCallback, v_unmap_cb, 0);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
 	 */
 	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Version"); n++;
+	XtSetArg (args[n], XmNtitle, "xephem About"); n++;
 	XtSetValues (XtParent(v_w), args, n);
 
 	/* make text widget for the version info */
@@ -310,7 +461,7 @@ v_makew()
 	ok_w = XmCreatePushButton (v_w, "VOk", args, n);
 	XtManageChild (ok_w);
 	XmStringFree (str);
-	set_something (v_w, XmNdefaultButton, (char *)ok_w);
+	set_something (v_w, XmNdefaultButton, (XtArgVal)ok_w);
 
 	/* make a frame for the drawing area */
 
@@ -330,7 +481,7 @@ v_makew()
 	/* make a drawing area for drawing the solar system */
 
 	n = 0;
-	vda_w = XmCreateDrawingArea (frame_w, "VersionMap", args, n);
+	vda_w = XmCreateDrawingArea (frame_w, "AboutMap", args, n);
 	XtAddCallback (vda_w, XmNexposeCallback, v_da_exp_cb, 0);
 	XtAddCallback (vda_w, XmNresizeCallback, v_da_exp_cb, 0);
 	XtManageChild (vda_w);
@@ -582,8 +733,8 @@ v_define_fgc()
 	Pixel fg, bg;
 
 	gcm = GCForeground | GCFunction;
-	get_something (vda_w, XmNforeground, (char *)&fg);
-	get_something (vda_w, XmNbackground, (char *)&bg);
+	get_something (vda_w, XmNforeground, (XtArgVal)&fg);
+	get_something (vda_w, XmNbackground, (XtArgVal)&bg);
 	gcv.foreground = fg ^ bg;
 	gcv.function = GXxor;
 	v_fgc = XCreateGC (dsp, win, gcm, &gcv);
